set CR [binary format H2 0D]
set LF [binary format H2 0A]
set CR_LF $CR$LF
set CONTROL_J [binary format H2 0A]
set CONTROL_F [binary format H2 06]
set CONTROL_A [binary format H2 01]
set ESC	[binary format H2 1B]

# 0 is Sun ??
set DAY_OF_WEEK {00 01 02 03 04 05 06}


# keeping track of the mode: "data", "program", or "calibrate"
set NO_MODE "no_mode"
set DATA_MODE "data"
set PROGRAM_MODE "program"
set CALIBRATE_MODE "calibrate"

set mode $NO_MODE			



# open serial communication
proc open_com {} {
	global serial
	set serial [open com1: RDWR]
	fconfigure $serial -mode "300,e,7,1"
	fconfigure $serial -blocking 1 -buffering none -buffersize 0 -translation binary
	# should put in blocking mode to get back the value read?? --> should have timeout mechanism??	
}

# close serial communication
proc close_com {} {
	global serial
	close $serial 
}



# send a byte to serial 
proc send_com {b} {
	global serial	
	puts -nonewline  $serial $b
}

# read from serial until CRLF is encountered
proc read_com {} {
	global serial	
	catch {gets $serial} ret 
		
	# delete the CR at the beginning or the end
	return [string trim $ret]
}


# initialize connection with the meter
proc initialize_con {} {
	global CR_LF
	global NO_MODE
	
	exit_program_mode

	after 1000
	exit_calibration_mode

	close_com
	open_com

	# SET UP 
	send_com "/?!"
	send_com $CR_LF
	
	set ret [read_com]
	puts $ret
	
	# expected returned setup code
	set SETUP_CODE "/RS4RESPEM"
	
	if {"$ret" == "$SETUP_CODE"} {
		puts "Setup OK"
	} else {
		puts "Setup Error"
	}
	set mode $NO_MODE
}

# eliminate the brackets 
proc strip_brackets {s} {
	#set firstpos [string last ( $s]
	#set lastpos  [string last ) $s]
	#return [string range $s [expr $firstpos+1] [expr $lastpos-1]]	
	
	set r ""
			
	regexp {.*\((.*)\).*} $s match r
	return $r
}

##################### DATA COLLECTION MODE #################
# collect data and return as global variables
proc collect_data {} {
	global DATA_MODE
	global mode
	
	########	
	global	unit_code 	
	global	type_meter      	
	global	max_demand	
	global	total_energy     
	global	rtc_sec 	
	global	rtc_min 	
	global	rtc_hour 	
	global 	rtc_time
	global	rtc_date 		
	global  rtc_day 	
	global  rtc_dayweek 	
	global  rtc_mon 	
	global  rtc_year 	
	global  rtc_date

	global  total_energy_back1
	global  active_energy_tar1_back1	
	global  active_energy_tar2_back1	
	global  active_energy_tar3_back1	
	global  active_energy_tar4_back1  
	global  apparent_energy_tar1_back1
	global  apparent_energy_tar2_back1
	global  apparent_energy_tar3_back1
	global  apparent_energy_tar4_back1
	global  max_demand_back1		
	global  type_reset_back1		
	global  sec_reset_back1	        	
	global  min_reset_back1	        	
	global  hour_reset_back1		
	global  date_reset_back1	  
	global  day_week_reset_back1	  
	global  month_reset_back1	  
	global  year_reset_back1	  
	global  reset_date_back1
	global  reset_time_back1

	global  total_energy_back2
	global  active_energy_tar1_back2	
	global  active_energy_tar2_back2	
	global  active_energy_tar3_back2	
	global  active_energy_tar4_back2  
	global  apparent_energy_tar1_back2
	global  apparent_energy_tar2_back2
	global  apparent_energy_tar3_back2
	global  apparent_energy_tar4_back2
	global  max_demand_back2		
	global  type_reset_back2		
	global  sec_reset_back2	        	
	global  min_reset_back2	        	
	global  hour_reset_back2		
	global  date_reset_back2	  
	global  day_week_reset_back2	  
	global  month_reset_back2	  
	global  year_reset_back2	  
	global  reset_date_back2
	global  reset_time_back2		        
	
	global  total_energy_back3
	global  active_energy_tar1_back3	
	global  active_energy_tar2_back3	
	global  active_energy_tar3_back3	
	global  active_energy_tar4_back3  
	global  apparent_energy_tar1_back3
	global  apparent_energy_tar2_back3
	global  apparent_energy_tar3_back3
	global  apparent_energy_tar4_back3
	global  max_demand_back3		
	global  type_reset_back3		
	global  sec_reset_back3	        	
	global  min_reset_back3	        	
	global  hour_reset_back3		
	global  date_reset_back3	  
	global  day_week_reset_back3	  
	global  month_reset_back3	  
	global  year_reset_back3	  
	global  reset_date_back3
	global  reset_time_back3		
	
	global  total_energy_back4
	global  active_energy_tar1_back4	
	global  active_energy_tar2_back4	
	global  active_energy_tar3_back4	
	global  active_energy_tar4_back4  
	global  apparent_energy_tar1_back4
	global  apparent_energy_tar2_back4
	global  apparent_energy_tar3_back4
	global  apparent_energy_tar4_back4
	global  max_demand_back4		
	global  type_reset_back4		
	global  sec_reset_back4	        	
	global  min_reset_back4	        	
	global  hour_reset_back4		
	global  date_reset_back4	  
	global  day_week_reset_back4	  
	global  month_reset_back4	  
	global  year_reset_back4	  
	global  reset_date_back4
	global  reset_time_back4		
	
	global  total_energy_back5
	global  active_energy_tar1_back5	
	global  active_energy_tar2_back5	
	global  active_energy_tar3_back5	
	global  active_energy_tar4_back5  
	global  apparent_energy_tar1_back5
	global  apparent_energy_tar2_back5
	global  apparent_energy_tar3_back5
	global  apparent_energy_tar4_back5
	global  max_demand_back5		
	global  type_reset_back5		
	global  sec_reset_back5	        	
	global  min_reset_back5	        	
	global  hour_reset_back5		
	global  date_reset_back5	  
	global  day_week_reset_back5	  
	global  month_reset_back5	  
	global  year_reset_back5	  
	global  reset_date_back5
	global  reset_time_back5	
	
	global  total_energy_back6
	global  active_energy_tar1_back6	
	global  active_energy_tar2_back6	
	global  active_energy_tar3_back6	
	global  active_energy_tar4_back6  
	global  apparent_energy_tar1_back6
	global  apparent_energy_tar2_back6
	global  apparent_energy_tar3_back6
	global  apparent_energy_tar4_back6
	global  max_demand_back6		
	global  type_reset_back6		
	global  sec_reset_back6	        	
	global  min_reset_back6	        	
	global  hour_reset_back6		
	global  date_reset_back6	  
	global  day_week_reset_back6	  
	global  month_reset_back6	  
	global  year_reset_back6	  
	global  reset_date_back6
	global  reset_time_back6		

	#########
	global serial	
	global CONTROL_F
	global CR_LF
	global CR
	global LF
	global result_data
	global DAY_OF_WEEK
	
	set mode $DATA_MODE
	
	# switch to data collection mode
	send_com $CONTROL_F
	
	send_com "0"
	
	send_com "0"
	
	send_com "0"
		
	send_com $CR
	
		
	#tk_messageBox -title Input_Invalid -message "$CR_LF " -icon warning
	
	send_com $LF
	

	
	# switch port 4800
	#close_com
	#set serial [open com1: RDWR]
	#fconfigure $serial -mode "4800,e,7,1" -blocking 1  -translation binary -buffering none
	#fconfigure $serial -blocking 1 -buffering none -translation binary
	
	
	
	# collect data
	
	
	
	set 	unit_code			[read_com]
	set 	type_meter			[read_com]	
	set 	rtc_data			[read_com]
	set 	unknown_H			[read_com]
	set 	unknown_R			[read_com]
	set	max_demand			[read_com]
	set 	total_energy 		[read_com]
	set 	A1				[read_com]
	set 	P1				[read_com]
	set 	M1				[read_com]
	set 	R1				[read_com]
	
	set 	A2				[read_com]
	set 	P2				[read_com]
	set 	M2				[read_com]
	set 	R2				[read_com]
	
	set 	A3				[read_com]
	set 	P3				[read_com]
	set 	M3				[read_com]
	set 	R3				[read_com]

	set 	A4				[read_com]
	set 	P4				[read_com]
	set 	M4				[read_com]
	set 	R4				[read_com]

	set 	A5				[read_com]
	set 	P5				[read_com]
	set 	M5				[read_com]
	set 	R5				[read_com]

	set 	A6				[read_com]
	set 	P6				[read_com]
	set 	M6				[read_com]
	set 	R6				[read_com]
	
	# switch 300
	
	fconfigure $serial -mode "300,e,7,1" -blocking 1  -translation binary -buffering none
	
	######## Strip Brackets	###########
	set 	unit_code 				[strip_brackets  	$unit_code	]	
	set 	type_meter 				[strip_brackets 	$type_meter	]	
	set 	rtc_data 				[strip_brackets 	$rtc_data	]
	set 	max_demand				[strip_brackets 	$max_demand	]	
	set 	total_energy 			[strip_brackets  	$total_energy]	
	
	  set 	A1				[strip_brackets $A1]
        set 	P1				[strip_brackets $P1]
        set 	M1				[strip_brackets $M1]
        set 	R1				[strip_brackets $R1]
                                                                        
	  set 	A2				[strip_brackets $A2]
        set 	P2				[strip_brackets $P2]
        set 	M2				[strip_brackets $M2]
        set 	R2				[strip_brackets $R2]
                                                                        
	  set 	A3				[strip_brackets $A3]
        set 	P3				[strip_brackets $P3]
        set 	M3				[strip_brackets $M3]
        set 	R3				[strip_brackets $R3]
                                                                        
	  set 	A4				[strip_brackets $A4]
        set 	P4				[strip_brackets $P4]
        set 	M4				[strip_brackets $M4]
        set 	R4				[strip_brackets $R4]
                                                                        
	  set 	A5				[strip_brackets $A5]
        set 	P5				[strip_brackets $P5]
        set 	M5				[strip_brackets $M5]
        set 	R5				[strip_brackets $R5]
                                                                        
	  set 	A6				[strip_brackets $A6]
        set 	P6				[strip_brackets $P6]
        set 	M6				[strip_brackets $M6]
        set 	R6				[strip_brackets $R6]
	
	#######################################################
	set	rtc_sec 			[lindex   $rtc_data		0 ]
	set	rtc_min 			[lindex   $rtc_data		1 ]
	set	rtc_hour 			[lindex   $rtc_data		2 ]	
	set	rtc_day 			[lindex   $rtc_data		3 ]
	set	rtc_dayweek 			[lindex   $rtc_data		4 ]		
	set	rtc_mon 			[lindex   $rtc_data		5 ]
	set	rtc_year 			[lindex   $rtc_data		6 ]

        # how about index number 4 of A1 ??
        # lacks total cumulative enegry    in R1??
        
	set     total_energy_back1              [lindex   $A1			0 ]
	set	active_energy_tar1_back1	[lindex   $A1		 	1 ]
	set     active_energy_tar2_back1	[lindex   $A1		        2 ]                                                            
	set     active_energy_tar3_back1	[lindex   $A1		        3 ]
	set     active_energy_tar4_back1        [lindex   $A1		        4 ]		
	set	apparent_energy_tar1_back1	[lindex   $P1		 	0 ]
	set     apparent_energy_tar2_back1	[lindex   $P1		        1 ]                                                            
	set     apparent_energy_tar3_back1	[lindex   $P1		        2 ]
	set     apparent_energy_tar4_back1      [lindex   $P1		        3 ]	
	set	max_demand_back1			  $M1             	
	set	type_reset_back1		[lindex   $R1		 	0 ]
	set     sec_reset_back1	        	[lindex   $R1		        1 ]
	set     min_reset_back1	        	[lindex   $R1		        2 ]
	set     hour_reset_back1		[lindex   $R1		        3 ]
	set	date_reset_back1	        [lindex   $R1		 	4 ]
	set     day_week_reset_back1	        [lindex   $R1		        5 ]
	set     month_reset_back1	        [lindex   $R1		        6 ]
	set     year_reset_back1	        [lindex   $R1		        7 ]                                 
	set 	total_energy_back2		[lindex   $A2		 	0 ]                              
	set	active_energy_tar1_back2	[lindex   $A2		 	1 ]
	set     active_energy_tar2_back2	[lindex   $A2		        2 ]
	set     active_energy_tar3_back2	[lindex   $A2		        3 ]
	set     active_energy_tar4_back2	[lindex   $A2		        4 ]
	set     apparent_energy_tar1_back2	[lindex   $P2		 	0 ]
	set     apparent_energy_tar2_back2	[lindex   $P2		        1 ]
	set     apparent_energy_tar3_back2	[lindex   $P2		        2 ]
	set     apparent_energy_tar4_back2	[lindex   $P2		        3 ]
	set     max_demand_back2			  $M2             	   
	set     type_reset_back2		[lindex   $R2		 	0 ]
	set     sec_reset_back2			[lindex   $R2		        1 ]
	set     min_reset_back2			[lindex   $R2		        2 ]
	set     hour_reset_back2		[lindex   $R2		        3 ]
	set     date_reset_back2		[lindex   $R2		 	4 ]
	set     day_week_reset_back2		[lindex   $R2		        5 ]
	set     month_reset_back2		[lindex   $R2		        6 ]
	set     year_reset_back2		[lindex   $R2		        7 ]
	set 	total_energy_back3	        [lindex   $A3		 	0 ]
	set	active_energy_tar1_back3	[lindex   $A3		 	1 ]
	set     active_energy_tar2_back3	[lindex   $A3		        2 ]
	set     active_energy_tar3_back3	[lindex   $A3		        3 ]
	set     active_energy_tar4_back3	[lindex   $A3		        4 ]
	set     apparent_energy_tar1_back3	[lindex   $P3		 	0 ]
	set     apparent_energy_tar2_back3	[lindex   $P3		        1 ]
	set     apparent_energy_tar3_back3	[lindex   $P3		        2 ]
	set     apparent_energy_tar4_back3	[lindex   $P3		        3 ]
	set     max_demand_back3			  $M3             	   
	set     type_reset_back3		[lindex   $R3		 	0 ]
	set     sec_reset_back3			[lindex   $R3		        1 ]
	set     min_reset_back3			[lindex   $R3		        2 ]
	set     hour_reset_back3		[lindex   $R3		        3 ]
	set     date_reset_back3		[lindex   $R3		 	4 ]
	set     day_week_reset_back3		[lindex   $R3		        5 ]
	set     month_reset_back3		[lindex   $R3		        6 ]                                         
	set     year_reset_back3		[lindex   $R3		        7 ]
	set	total_energy_back4		[lindex   $A4		 	0 ]	       	
	set	active_energy_tar1_back4	[lindex   $A4		 	1 ]
	set     active_energy_tar2_back4	[lindex   $A4		        2 ]
	set     active_energy_tar3_back4	[lindex   $A4		        3 ]
	set     active_energy_tar4_back4	[lindex   $A4		        4 ]
	set     apparent_energy_tar1_back4	[lindex   $P4		 	0 ]
	set     apparent_energy_tar2_back4	[lindex   $P4		        1 ]
	set     apparent_energy_tar3_back4	[lindex   $P4		        2 ]
	set     apparent_energy_tar4_back4	[lindex   $P4		        3 ]
	set     max_demand_back4			  $M4             	   
	set     type_reset_back4		[lindex   $R4		 	0 ]
	set     sec_reset_back4			[lindex   $R4		        1 ]
	set     min_reset_back4			[lindex   $R4		        2 ]
	set     hour_reset_back4		[lindex   $R4		        3 ]
	set     date_reset_back4		[lindex   $R4		 	4 ]
	set     day_week_reset_back4		[lindex   $R4		        5 ]
	set     month_reset_back4		[lindex   $R4		        6 ]
	set     year_reset_back4		[lindex   $R4		        7 ]	   
	set 	total_energy_back5		[lindex   $A5		 	0 ]   
	set     active_energy_tar1_back5	[lindex   $A5		 	1 ]
	set     active_energy_tar2_back5	[lindex   $A5		        2 ]
	set     active_energy_tar3_back5	[lindex   $A5		        3 ]
	set     active_energy_tar4_back5	[lindex   $A5		        4 ]
	set     apparent_energy_tar1_back5	[lindex   $P5		 	0 ]
	set     apparent_energy_tar2_back5	[lindex   $P5		        1 ]
	set     apparent_energy_tar3_back5	[lindex   $P5		        2 ]
	set	apparent_energy_tar4_back5	[lindex   $P5		        3 ]
	set	max_demand_back5			  $M5             	         
	set     type_reset_back5		[lindex   $R5		 	0 ]
	set	sec_reset_back5			[lindex   $R5		        1 ]
	set	min_reset_back5			[lindex   $R5		        2 ]
	set	hour_reset_back5		[lindex   $R5		        3 ]
	set     date_reset_back5		[lindex   $R5		 	4 ]
	set	day_week_reset_back5		[lindex   $R5		        5 ]
	set	month_reset_back5		[lindex   $R5		        6 ]
	set	year_reset_back5		[lindex   $R5		        7 ]       
	set	total_energy_back6		[lindex   $A6		 	0 ]        
	set	active_energy_tar1_back6	[lindex   $A6		 	1 ]
	set	active_energy_tar2_back6	[lindex   $A6		        2 ]
	set	active_energy_tar3_back6	[lindex   $A6		        3 ]
	set	active_energy_tar4_back6	[lindex   $A6		        4 ]
	set	apparent_energy_tar1_back6	[lindex   $P6		 	0 ]
	set     apparent_energy_tar2_back6	[lindex   $P6		        1 ]
	set	apparent_energy_tar3_back6	[lindex   $P6		        2 ]
	set	apparent_energy_tar4_back6	[lindex   $P6		        3 ]
	set	max_demand_back6			  $M6             	   
	set     type_reset_back6		[lindex   $R6		 	0 ]         
	set	sec_reset_back6			[lindex   $R6		        1 ]           
	set	min_reset_back6			[lindex   $R6		        2 ]           
	set	hour_reset_back6		[lindex   $R6		        3 ]           
	set     date_reset_back6		[lindex   $R6		 	4 ]         
	set	day_week_reset_back6		[lindex   $R6		        5 ]           
	set	month_reset_back6		[lindex   $R6		        6 ]           
	set	year_reset_back6		[lindex   $R6		        7 ]       		
		
		#convert the day of week
	set 	rtc_dayweek			[lindex   $DAY_OF_WEEK		$rtc_dayweek]	
	set 	rtc_date			${rtc_day}-${rtc_mon}-${rtc_year}(${rtc_dayweek})
	set 	rtc_time			${rtc_hour}:${rtc_min}:${rtc_sec}
	set	day_week_reset_back1		[lindex   $DAY_OF_WEEK		$day_week_reset_back1]	
	set 	reset_date_back1		${date_reset_back1}-${month_reset_back1}-${year_reset_back1}(${day_week_reset_back1})
	set 	reset_time_back1		${hour_reset_back1}:${min_reset_back1}:${sec_reset_back1}
	set	day_week_reset_back2		[lindex   $DAY_OF_WEEK		$day_week_reset_back2]	
	set 	reset_date_back2		${date_reset_back2}-${month_reset_back2}-${year_reset_back2}(${day_week_reset_back2})
	set 	reset_time_back2		${hour_reset_back2}:${min_reset_back2}:${sec_reset_back2}
	set	day_week_reset_back3		[lindex   $DAY_OF_WEEK		$day_week_reset_back3]	
	set 	reset_date_back3		${date_reset_back3}-${month_reset_back3}-${year_reset_back3}(${day_week_reset_back3})
	set 	reset_time_back3		${hour_reset_back3}:${min_reset_back3}:${sec_reset_back3}
	set	day_week_reset_back4		[lindex   $DAY_OF_WEEK		$day_week_reset_back4]	
	set 	reset_date_back4		${date_reset_back4}-${month_reset_back1}-${year_reset_back4}(${day_week_reset_back4})
	set 	reset_time_back4		${hour_reset_back4}:${min_reset_back1}:${sec_reset_back4}		
	set	day_week_reset_back5		[lindex   $DAY_OF_WEEK		$day_week_reset_back5]	
	set 	reset_date_back5		${date_reset_back5}-${month_reset_back5}-${year_reset_back5}(${day_week_reset_back5})
	set 	reset_time_back5		${hour_reset_back5}:${min_reset_back5}:${sec_reset_back5}			
	set	day_week_reset_back6		[lindex   $DAY_OF_WEEK		$day_week_reset_back6]	
	set 	reset_date_back6		${date_reset_back6}-${month_reset_back6}-${year_reset_back6}(${day_week_reset_back6})
	set 	reset_time_back6		${hour_reset_back6}:${min_reset_back6}:${sec_reset_back6}			
		
}    

################# PROGAMMING MODE ###################	  	        	        
# switch to programming mode
proc switch_program_mode {} {
	global CONTROL_F
	global CONTROL_A
	global CR_LF
	global PROGRAM_MODE
	global mode
	
	send_com $CONTROL_F
	send_com 001
	send_com $CR_LF
	
	set ret [read_com] 
	puts $ret
	
	
	###
	set PROG_CODE ${CONTROL_A}P0(RENSPEM)
	
	###		
	
	if {"$PROG_CODE" == "$ret"} {
		puts "Correct received message. Sending password..."		
		
		# send password
		set PASSWORD ${CONTROL_A}P1(SPEM)
		
		send_com $PASSWORD			
		
		# the CR_LF
		set ret [read_com] 
		# the ACK/NACK		
		set ret [read_com] 		
		set ret [scan $ret %c] 
		#puts $ret
		
		#The NACK is returned without CRLF --> need to read character (not line) from the serial stream
		#dunno whether TCL can do??
		if {  06 == $ret} {
			puts "Password acked"		
			
		} else {
			puts "Password not acked"
		}
	
	} else {
		puts "Incorrect receive message"
	}
	
	set mode $PROGRAM_MODE
}

#exit programming mode
proc exit_program_mode {} {
	global CONTROL_A
	global CR
	global NO_MODE
	global mode		
		
	send_com $CONTROL_A
	send_com B0
	send_com $CR
	
	set mode $NO_MODE
}

# update RAM
proc update_ram {} {
	global CONTROL_A	
	send_com $CONTROL_A
	send_com W1O()		
}

# send write command
# if no error occurs --> return empty string
# if error occurs --> return the error string
proc send_write_command {comm} {
	global CONTROL_A
	send_com $CONTROL_A
	for {set i 0} {$i < [llength $comm]} {incr i} {
		send_com [lindex $comm	$i]
		after 500
	}
		
	# read back returned value
	set ret [read_com]
	
	if {[string length $ret] != 0} {		
		# error occurs
		# read the CRLF in 		
		read_com
	}	
	return $ret
}

# update scroll period
# p : 10, 20, or 30
proc write_scroll_period {p} {		
	set ret [send_write_command W1S(${p})]
	if {[string length $ret] != 0} {		
		# error occurs
		# read the CRLF in 		
		read_com
		puts $ret
	}			
}

# update unit code
# ucode : 6 alphanumerics
proc write_unit_code {ucode} {
	set ret [send_write_command W1U(${ucode})]	
	puts $ret 	
}

# update MD Integration period programming
# mdp : 30 or 60
proc write_md_period {mdp} {
	set ret [send_write_command W1I(${mdp})]	
	puts $ret 
}


#update  RTC
# t : seconds minutes hour date dayoweek month year
proc write_rtc {t} {
	set ret [send_write_command W1R(${t})]
	puts $ret 
}

# update tariffs
proc write_tariffs {tar} {
	set ret [send_write_command W1T(${tar})]
	puts $ret 	
		
}

#send read command and return value
proc send_read_command {comm} {
	global CONTROL_A
	send_com $CONTROL_A
	send_com $comm
	
	set ret [read_com]
	
	# strip out the brackets	 	
	return [strip_brackets $ret]
	
}	
# read scroll period
proc read_scroll_period {} {
	return [send_read_command R1s()]
}	

# read unit code
proc read_unit_code {} {
	return [send_read_command R1u()]
}

# read md integration period
proc read_md_period {} {
	return [send_read_command R1i()]
}

# read rtc
proc read_rtc {} {
	return [send_read_command R1r()]
}

# read tariffs
proc read_tariffs {} {
	return [send_read_command R1t()]
}

######################### CALIBRATION MODE ##########################
proc switch_calibration_mode {} {
	global CONTROL_F
	global CR_LF
	global CR
	global CALIBRATE_MODE
	global mode
	
	send_com $CONTROL_F
	send_com 004
	send_com $CR_LF
	
	# send password
	set PASSWORD SPEM		
	send_com $PASSWORD			
	send_com $CR				
				
	
	set ret [read_com] 		
	
	# if password is correct --> Calibration Mode is returned
	#The NACK is returned without CRLF --> need to read character (not line) from the serial stream
	#dunno whether TCL can do??
	
	set mode $CALIBRATE_MODE
		
	puts $ret
			
}


# exit calibration mode
proc exit_calibration_mode {} {
	global ESC
	send_com $ESC
	global NO_MODE
	global mode
	
	set mode $NO_MODE	
}		


#read the voltage and the gain
proc read_voltage_coefficient {} {
	# wait for 5 seconds and send V (0x56)
	send_com V			
	# read in the "Voltage Calibration"
	read_com
	# read in the "V:..."
	set v [read_com]
	#CRLF
	read_com
	#Gain ...
	set g [read_com]

	

	set v  [lindex [regsub {:} $v { }] 1]
	set g  [lindex $g 1]	
	
	# return both 
	return "$v $g"	
}	

#read the voltage and the gain
proc read_voltage_coefficient1 {} {
	# wait for 5 seconds and send V (0x56)
	send_com V			
	# read in the "Voltage Calibration"
	read_com

	# read in the "V:..."
	set v [read_com]

	
	#Gain ...
	set g [read_com]

	#Phase and Neutral currents
	set h [read_com]
	set j [read_com]

	set v  [lindex [regsub {:} $v { }] 1]
	set g  [lindex $g 1]	

	
	# return both 
	return "$v $h $j $g"	
}	


#voltage calibration - change coefficient 
# ce : coefficient
proc calibrate_voltage_coefficient {ce} {
	global CR
	
	send_com C0
	after 500
	send_com $ce
	after 500
	send_com $CR
	
	# read back the message "calibrated 0"
	set ret [read_com]
	puts $ret	
}	

#read gain calibration  
proc read_gain_coefficient {} {
	global CR
	
	# give appropriate current and voltage
	send_com $CR
	
	# wait for 1 min 
	# "Test In Progress"
	
	after [expr 1000*60]
	
	send_com $CR		
			
	# CRLF
	read_com
	
	# read back the energy 
	set e [read_com]	
	set e [lindex [regsub {=} $e { }] 1]
	return $e
}	

#gain calibration - change coefficient 
#ce:  coefficient
#c : C1, C2, ..., C8
proc calibrate_gain_coefficient {c ce} {
	global CR
	
	send_com $c
	after 500
	send_com $ce
	after 500
	send_com $CR
	
	# read back the message "calibrated"
	set ret [read_com]
	puts $ret			
}

#phase calibration - change coefficient 
#ce : coefficient
#c  : A1, A2, ..., A8
proc calibrate_phase_coefficient {a ce} {
	global CR
	
	send_com $a
	after 500
	send_com $ce
	after 500
	send_com $CR
	
	# read back the message "calibrated"
	set ret [read_com]
	puts $ret
	
}

# read all the coefficients
proc read_coefficients {} {
	global serial
	
	send_com R	
	#the CRLF
	read_com
	
	set ret [list]
	
	for {set i 0} {$i < 16} {incr i} {
		# Range...
		puts [read_com]		
		#Gain ...
		set t [read_com]
				
		lappend ret [lindex $t 1]					
	}			
	
	# Range ...
	
	# return the list of values		
	return $ret
}

#################################
#close any mode if it is in 
proc exit_all_modes {} {
	global mode
	global PROGRAM_MODE
	global CALIBRATE_MODE
	
	if {$mode == $PROGRAM_MODE} {
		exit_program_mode
	}  elseif {$mode == $CALIBRATE_MODE} {
		exit_calibration_mode
	}	
}
##################################
# close any mode if it is in and reconnect COM port + initialize connection
proc reinitialize_con {} {	
	exit_all_modes
	close_com
	open_com
	initialize_con	
}

####################################
# useful functions

#check for valid time
# t : "hh:mm:ss" or "hh:mm"
proc check_valid_time {t} {			
	if {[regexp {([0-9]+):([0-9]+)(:([0-9]+))?} [string trim $t] match hh mm ss_colon ss ]} {
		if {$ss == ""} { set ss 0 }
		return [expr ($hh<=23)&&($mm<=59)&&($ss<=59)]		
	} else { return 0 }
}

# check for valid date
# d : dd/mm/yyyy

##################################                
proc check_valid_date {d} {
	
	if {[regexp {([0-9]+)/([0-9]+)/([0-9]+)?} [string trim $d] match dd mon yy]} {
		
		set is_leap [expr ($yy%4==0) && ($yy%100!=0 || $yy%400==0)]
				
		if { $is_leap==0 && $mon==2 && $dd>28} { return 0 }
		if { $is_leap==1 && $mon==2 && $dd>29} { return 0 }
		
		if { $dd > 31 || $mon > 12 || $yy < 1900 || $yy > 2100 }  { return 0 }
				
		if { ($mon==4 || $mon==6 || $mon==9 || $mon==11) && ($dd > 30) }  { return 0 }
		
		return 1
	} else { return 0 }		
} 

open_com                       
#initialize_con  

# should have reconnect proc which will exit all modes and reconnect COM ??


#collect_data    
                
#switch_program_mode
#read_scroll_period
#read_unit_code
#read_md_period
#read_rtc
#read_tariffs
            
#switch_calibration_mode                                                                


#close_com      
                
                
                
