#####################################################################
# Files for logging purpose will be created in user's temp directory
# 

#console show
source comm.tcl

###################################
# Create a new notebook widget
#
proc Notebook:create {w args} {
  global Notebook
  set Notebook($w,width) 400
  set Notebook($w,height) 300
  set Notebook($w,pages) {}
  set Notebook($w,top) 0
  set Notebook($w,pad) 5
  set Notebook($w,fg,on) black
  set Notebook($w,fg,off) grey50
  canvas $w -bd 0 -highlightthickness 0 -takefocus 0
  set Notebook($w,bg) [$w cget -bg]
  bind $w <1> "Notebook:click $w %x %y"
  bind $w <Configure> "Notebook:scheduleExpand $w"
  eval Notebook:config $w $args
}

#######################################################
# Change configuration options for the notebook widget
#
proc Notebook:config {w args} {
  global Notebook
  foreach {tag value} $args {
    switch -- $tag {
      -width {
        set Notebook($w,width) $value
      }
      -height {
        set Notebook($w,height) $value
      }
      -pages {
        set Notebook($w,pages) $value
      }
      -pad {
        set Notebook($w,pad) $value
      }
      -bg {
        set Notebook($w,bg) $value
      }
      -fg {
        set Notebook($w,fg,on) $value
      }
      -disabledforeground {
        set Notebook($w,fg,off) $value
      }
    }
  }

  #################################################################
  # After getting new configuration values, reconstruct the widget
  #
  $w delete all
  set Notebook($w,x1) $Notebook($w,pad)
  set Notebook($w,x2) [expr $Notebook($w,x1)+2]
  set Notebook($w,x3) [expr $Notebook($w,x2)+$Notebook($w,width)]
  set Notebook($w,x4) [expr $Notebook($w,x3)+2]
  set Notebook($w,y1) [expr $Notebook($w,pad)+2]
  set Notebook($w,y2) [expr $Notebook($w,y1)+2]
  set Notebook($w,y5) [expr $Notebook($w,y1)+30]
  set Notebook($w,y6) [expr $Notebook($w,y5)+2]
  set Notebook($w,y3) [expr $Notebook($w,y6)+$Notebook($w,height)]
  set Notebook($w,y4) [expr $Notebook($w,y3)+2]
  set x $Notebook($w,x1)
  set cnt 0
  set y7 [expr $Notebook($w,y1)+10]
  foreach p $Notebook($w,pages) {
    set Notebook($w,p$cnt,x5) $x
    set id [$w create text 0 0 -text $p -anchor nw -tags "p$cnt t$cnt"]
    set bbox [$w bbox $id]
    set width [lindex $bbox 2]
    $w move $id [expr $x+10] $y7
    $w create line \
       $x $Notebook($w,y5)\
       $x $Notebook($w,y2) \
       [expr $x+2] $Notebook($w,y1) \
       [expr $x+$width+16] $Notebook($w,y1) \
       -width 2 -fill white -tags p$cnt
    $w create line \
       [expr $x+$width+16] $Notebook($w,y1) \
       [expr $x+$width+18] $Notebook($w,y2) \
       [expr $x+$width+18] $Notebook($w,y5) \
       -width 2 -fill black -tags p$cnt
    set x [expr $x+$width+20]
    set Notebook($w,p$cnt,x6) [expr $x-2]
    if {![winfo exists $w.f$cnt]} {
      frame $w.f$cnt -bd 0
    }
    $w.f$cnt config -bg $Notebook($w,bg)
    place $w.f$cnt -x $Notebook($w,x2) -y $Notebook($w,y6) \
      -width $Notebook($w,width) -height $Notebook($w,height)
    incr cnt
  }
  $w create line \
     $Notebook($w,x1) [expr $Notebook($w,y5)-2] \
     $Notebook($w,x1) $Notebook($w,y3) \
     -width 2 -fill white
  $w create line \
     $Notebook($w,x1) $Notebook($w,y3) \
     $Notebook($w,x2) $Notebook($w,y4) \
     $Notebook($w,x3) $Notebook($w,y4) \
     $Notebook($w,x4) $Notebook($w,y3) \
     $Notebook($w,x4) $Notebook($w,y6) \
     $Notebook($w,x3) $Notebook($w,y5) \
     -width 2 -fill black
  $w config -width [expr $Notebook($w,x4)+$Notebook($w,pad)] \
            -height [expr $Notebook($w,y4)+$Notebook($w,pad)] \
            -bg $Notebook($w,bg)
  set top $Notebook($w,top)
  set Notebook($w,top) -1
  Notebook:raise.page $w $top
}

########################################################################
# This routine is called whenever the mouse-button is pressed over
# the notebook.  It determines if any page should be raised and raises
# that page.
#
proc Notebook:click {w x y} {
  global Notebook
  if {$y<$Notebook($w,y1) || $y>$Notebook($w,y6)} return
  set N [llength $Notebook($w,pages)]
  for {set i 0} {$i<$N} {incr i} {
    if {$x>=$Notebook($w,p$i,x5) && $x<=$Notebook($w,p$i,x6)} {
      Notebook:raise.page $w $i
      break
    }
  }
}

##################################################################
# For internal use only.  This procedure raised the n-th page of
# the notebook
#
proc Notebook:raise.page {w n} {
  global Notebook
  if {$n<0 || $n>=[llength $Notebook($w,pages)]} return
  set top $Notebook($w,top)
  if {$top>=0 && $top<[llength $Notebook($w,pages)]} {
    $w move p$top 0 2
  }
  $w move p$n 0 -2
  $w delete topline
  if {$n>0} {
    $w create line \
       $Notebook($w,x1) $Notebook($w,y6) \
       $Notebook($w,x2) $Notebook($w,y5) \
       $Notebook($w,p$n,x5) $Notebook($w,y5) \
       $Notebook($w,p$n,x5) [expr $Notebook($w,y5)-2] \
       -width 2 -fill white -tags topline
  }
  $w create line \
    $Notebook($w,p$n,x6) [expr $Notebook($w,y5)-2] \
    $Notebook($w,p$n,x6) $Notebook($w,y5) \
    -width 2 -fill white -tags topline
  $w create line \
    $Notebook($w,p$n,x6) $Notebook($w,y5) \
    $Notebook($w,x3) $Notebook($w,y5) \
    -width 2 -fill white -tags topline
  set Notebook($w,top) $n
  raise $w.f$n
}

####################################################################
# Change the page-specific configuration options for the notebook
#
proc Notebook:pageconfig {w name args} {
  global Notebook
  set i [lsearch $Notebook($w,pages) $name]
  if {$i<0} return
  foreach {tag value} $args {
    switch -- $tag {
      -state {
        if {"$value"=="disabled"} {
          $w itemconfig t$i -fg $Notebook($w,fg,off)
        } else {
          $w itemconfig t$i -fg $Notebook($w,fg,on)
        }
      }
      -onexit {
        set Notebook($w,p$i,onexit) $value
      }
    }
  }
}

####################################################################
# This procedure raises a notebook page given its name.  But first
# we check the "onexit" procedure for the current page (if any) and
# if it returns false, we don't allow the raise to proceed.
#
proc Notebook:raise {w name} {
  global Notebook
  set i [lsearch $Notebook($w,pages) $name]
  if {$i<0} return
  if {[info exists Notebook($w,p$i,onexit)]} {
    set onexit $Notebook($w,p$i,onexit)
    if {"$onexit"!="" && [eval uplevel #0 $onexit]!=0} {
      Notebook:raise.page $w $i
    }
  } else {
    Notebook:raise.page $w $i
  }
}

####################################################################
# Return the frame associated with a given page of the notebook.
#
proc Notebook:frame {w name} {
  global Notebook
  set i [lsearch $Notebook($w,pages) $name]
  if {$i>=0} {
    return $w.f$i
  } else {
    return {}
  }
}

################################################################
# Try to resize the notebook to the next time we become idle.
#
proc Notebook:scheduleExpand w {
  global Notebook
  if {[info exists Notebook($w,expand)]} return
  set Notebook($w,expand) 1
  after idle "Notebook:expand $w"
}

############################################################
# Resize the notebook to fit inside its containing widget.
#
proc Notebook:expand w {
  global Notebook
  set wi [expr [winfo width $w]-($Notebook($w,pad)*2+4)]
  set hi [expr [winfo height $w]-($Notebook($w,pad)*2+36)]
  Notebook:config $w -width $wi -height $hi
  catch {unset Notebook($w,expand)}
}

#############################################################
# The following code implements the Energy Meter Windows
# 
#
destroy .f1.l1
destroy .f1.l2
destroy .f1.l3
destroy .f1.l4
destroy .f1.l5
destroy .f1.l6
destroy .f1.l7
destroy .f1.l8
destroy .f1.l9
destroy .f1.l10
destroy .f1
Notebook:create .n -pages {Data_Collection Parameter_Programming Meter_Calibration} -pad 20 
pack .n -fill both -expand 1
set w [Notebook:frame .n Data_Collection]
set v [Notebook:frame .n Parameter_Programming]
set x [Notebook:frame .n Meter_Calibration]

Notebook:raise.page .n 1

#####################################
# Frame		: Frame 1 
#(content property, RTC, Present Energy)
#####################################
	frame $w.f1
	pack $w.f1 -side top -fill both -expand 0
	
	frame $w.f1.f1 
	pack $w.f1.f1 -side left -fill both -expand 0
	
	frame $w.f1.f2               
	pack $w.f1.f2 -side left -fill both -expand 0
	
	frame $w.f1.f3 
	pack $w.f1.f3 -side left -fill both -expand 0
		
	set property [labelframe $w.f1.f1.lf1 -text "Meter Properties" -font *-ansi-bold-r-*-12-*] 
	set rtc_data [labelframe $w.f1.f2.lf2 -text "RTC Data" -font *-ansi-bold-r-*-12-*] 
	set energy_data [labelframe $w.f1.f3.lf3 -text "Present Energy Data" -font *-ansi-bold-r-*-12-*] 
	pack $property $rtc_data $energy_data -side left -expand 0 -padx 3 -pady 3
	
	set property1 [frame $property.f1]
	set property2 [frame $property.f2]
	pack $property1 $property2 -side left -fill both -expand 1 -padx 3 -pady 3
	
	label $property1.l1 -text "Unit Code:"
	label $property1.l2 -text "Meter Type:"
	foreach i {1 2} {pack $property1.l$i -side top  -anchor nw}
	
	#dummy value for display only
	set unit_code "" 
	set meter_type ""
	label $property2.l1 -width 10 -text $unit_code -font *-ansi-bold-r-*-12-*
	label $property2.l2 -width 10 -text $meter_type -font *-ansi-bold-r-*-12-*
	foreach i {1 2} {pack $property2.l$i -side top  -anchor nw}

	set rtc_data1 [frame $rtc_data.f1]
	set rtc_data2 [frame $rtc_data.f2]	
	pack $rtc_data1 $rtc_data2 -side left -fill both -expand 1 -padx 3 -pady 3
		
	label $rtc_data1.l1 -text "RTC Date:"
	label $rtc_data1.l2 -text "RTC Time:"
	foreach i {1 2} {pack $rtc_data1.l$i -side top  -anchor nw}
	
	#dummy value for display only
	set rtc_day 	"" 
	set rtc_dd 	""
	set rtc_mm 	"" 
	set rtc_yy 	""
	set rtc_hour 	""
	set rtc_minute 	""
	set rtc_second 	""
	set rtc_time 	""
	set rtc_date	""
	
	label $rtc_data2.l1 -width 10 -text $rtc_date -font *-ansi-bold-r-*-12-*
	label $rtc_data2.l2 -width 15 -text $rtc_time -font *-ansi-bold-r-*-12-*
	foreach i {1 2} {pack $rtc_data2.l$i -side top  -anchor nw}
	
	set energy_data1 [frame $energy_data.f1]
	set energy_data2 [frame $energy_data.f2]
	pack $energy_data1 $energy_data2 -side left -fill both -expand 1 -padx 3 -pady 3
	
	label $energy_data1.l1 -text "Maximum Demand:"
	label $energy_data1.l2 -text "Total cumulative enrgy:"
	foreach i {1 2} {pack $energy_data1.l$i -side top  -anchor nw}
	
	set p_MD ""  
	set p_kwh ""
	label $energy_data2.l1 -width 10 -text $p_MD -font *-ansi-bold-r-*-12-*
	label $energy_data2.l2 -width 15 -text $p_kwh -font *-ansi-bold-r-*-12-*
	foreach i {1 2} {pack $energy_data2.l$i -side top  -fill both}

#####################################
# Frame		: Frame 3
#(Close Button and Retrieve button)
#####################################

	frame $w.f3
	pack $w.f3 -side bottom -fill both -expand 1
	
	bind . <Destroy> {
								exit_all_modes
								close_com
			}

	button $w.f3.b1 -width 11 -text "Close" -command {
								exit_all_modes
								close_com
								destroy .
							  }
	pack $w.f3.b1 -side right 	
	
	button $w.f3.b2 -text "Retrieve"  -width 11 -command {
				# retrieve values
								     
               			reinitialize_con		             
				collect_data  				              				

				# update GUI 
				
				$property2.l1 configure -text $unit_code
				$property2.l2 configure -text $type_meter
				
				$rtc_data2.l1 configure -text $rtc_date
                                $rtc_data2.l2 configure -text $rtc_time
                                
				$energy_data2.l1 configure -text $max_demand
				$energy_data2.l2 configure -text $total_energy				
				
				# backup 1
								
				$backup_1.f1.l1 configure -text $type_reset_back1
				$backup_1.f1.l2 configure -text $reset_time_back1				
				$backup_1.f1.l3 configure -text $reset_date_back1
				
					
				$backup_1.f1.l4 configure -text $total_energy_back1
				$backup_1.f1.l5 configure -text $active_energy_tar1_back1	
				$backup_1.f1.l6 configure -text $active_energy_tar2_back1	
				$backup_1.f1.l7 configure -text $active_energy_tar3_back1	
				$backup_1.f1.l8 configure -text $active_energy_tar4_back1        
				
				$backup_1.f1.l9  configure -text $apparent_energy_tar1_back1      
				$backup_1.f1.l10 configure -text $apparent_energy_tar2_back1      
				$backup_1.f1.l11 configure -text $apparent_energy_tar3_back1      
				$backup_1.f1.l12 configure -text $apparent_energy_tar4_back1      
				
				# backup 2
								
				$backup_2.f1.l1 configure -text $type_reset_back2
				$backup_2.f1.l2 configure -text $reset_time_back2				
				$backup_2.f1.l3 configure -text $reset_date_back2
				        
					
				$backup_2.f1.l4 configure -text $total_energy_back2
				$backup_2.f1.l5 configure -text $active_energy_tar1_back2	
				$backup_2.f1.l6 configure -text $active_energy_tar2_back2	
				$backup_2.f1.l7 configure -text $active_energy_tar3_back2	
				$backup_2.f1.l8 configure -text $active_energy_tar4_back2        
				        
				$backup_2.f1.l9  configure -text $apparent_energy_tar1_back2      
				$backup_2.f1.l10 configure -text $apparent_energy_tar2_back2      
				$backup_2.f1.l11 configure -text $apparent_energy_tar3_back2      
				$backup_2.f1.l12 configure -text $apparent_energy_tar4_back2      

				# backup 3
								
				$backup_3.f1.l1 configure -text $type_reset_back3
				$backup_3.f1.l2 configure -text $reset_time_back3				
				$backup_3.f1.l3 configure -text $reset_date_back3
				        
					
				$backup_3.f1.l4 configure -text $total_energy_back3
				$backup_3.f1.l5 configure -text $active_energy_tar1_back3	
				$backup_3.f1.l6 configure -text $active_energy_tar2_back3	
				$backup_3.f1.l7 configure -text $active_energy_tar3_back3	
				$backup_3.f1.l8 configure -text $active_energy_tar4_back3        
				        
				$backup_3.f1.l9  configure -text $apparent_energy_tar1_back3     
				$backup_3.f1.l10 configure -text $apparent_energy_tar2_back3     
				$backup_3.f1.l11 configure -text $apparent_energy_tar3_back3     
				$backup_3.f1.l12 configure -text $apparent_energy_tar4_back3     

				# backup 4
								
				$backup_4.f1.l1 configure -text $type_reset_back4
				$backup_4.f1.l2 configure -text $reset_time_back4				
				$backup_4.f1.l3 configure -text $reset_date_back4
				        
					
				$backup_4.f1.l4 configure -text $total_energy_back4
				$backup_4.f1.l5 configure -text $active_energy_tar1_back4	
				$backup_4.f1.l6 configure -text $active_energy_tar2_back4	
				$backup_4.f1.l7 configure -text $active_energy_tar3_back4	
				$backup_4.f1.l8 configure -text $active_energy_tar4_back4        
				        
				$backup_4.f1.l9  configure -text $apparent_energy_tar1_back4      
				$backup_4.f1.l10 configure -text $apparent_energy_tar2_back4      
				$backup_4.f1.l11 configure -text $apparent_energy_tar3_back4      
				$backup_4.f1.l12 configure -text $apparent_energy_tar4_back4      

				# backup 5
								
				$backup_5.f1.l1 configure -text $type_reset_back5
				$backup_5.f1.l2 configure -text $reset_time_back5				
				$backup_5.f1.l3 configure -text $reset_date_back5
				        
					
				$backup_5.f1.l4 configure -text $total_energy_back5
				$backup_5.f1.l5 configure -text $active_energy_tar1_back5	
				$backup_5.f1.l6 configure -text $active_energy_tar2_back5	
				$backup_5.f1.l7 configure -text $active_energy_tar3_back5	
				$backup_5.f1.l8 configure -text $active_energy_tar4_back5        
				        
				$backup_5.f1.l9  configure -text $apparent_energy_tar1_back5      
				$backup_5.f1.l10 configure -text $apparent_energy_tar2_back5      
				$backup_5.f1.l11 configure -text $apparent_energy_tar3_back5      
				$backup_5.f1.l12 configure -text $apparent_energy_tar4_back5      

				# backup 6
								
				$backup_6.f1.l1 configure -text $type_reset_back6
				$backup_6.f1.l2 configure -text $reset_time_back6				
				$backup_6.f1.l3 configure -text $reset_date_back6
				        
					
				$backup_6.f1.l4 configure -text $total_energy_back6
				$backup_6.f1.l5 configure -text $active_energy_tar1_back6	
				$backup_6.f1.l6 configure -text $active_energy_tar2_back6	
				$backup_6.f1.l7 configure -text $active_energy_tar3_back6	
				$backup_6.f1.l8 configure -text $active_energy_tar4_back6        
				        
				$backup_6.f1.l9  configure -text $apparent_energy_tar1_back6      
				$backup_6.f1.l10 configure -text $apparent_energy_tar2_back6      
				$backup_6.f1.l11 configure -text $apparent_energy_tar3_back6      
				$backup_6.f1.l12 configure -text $apparent_energy_tar4_back6      
				
	}				
				
				
				
	pack $w.f3.b2 -side right 
                                

#####################################
# Frame		: Frame 2
#(6 Backup Notebook)
#####################################
	
	frame $w.f2
	pack $w.f2 -side top -fill y -expand 0

	Notebook:create $w.f2.n -pages {Backup1 Backup2 Backup3 Backup4 Backup5 Backup6} -pad 20
	pack $w.f2.n -fill both -ipadx 100 -ipady 100
	
	set backup_1 [Notebook:frame $w.f2.n Backup1]
	set backup_2 [Notebook:frame $w.f2.n Backup2]
	set backup_3 [Notebook:frame $w.f2.n Backup3]
	set backup_4 [Notebook:frame $w.f2.n Backup4]
	set backup_5 [Notebook:frame $w.f2.n Backup5]
	set backup_6 [Notebook:frame $w.f2.n Backup6]		
	
	
	frame $backup_1.f2
	frame $backup_2.f2
	frame $backup_3.f2
	frame $backup_4.f2
	frame $backup_5.f2
	frame $backup_6.f2

	pack $backup_1.f2 -fill both -side left
	pack $backup_2.f2 -fill both -side left
	pack $backup_3.f2 -fill both -side left
	pack $backup_4.f2 -fill both -side left
	pack $backup_5.f2 -fill both -side left
	pack $backup_6.f2 -fill both -side left

	
	label $backup_1.f2.l1 -text 	"Type of Reset			:"
	label $backup_1.f2.l2 -text 	"Reset Time			:"
	label $backup_1.f2.l3 -text 	"Reset Date			:"
	label $backup_1.f2.l4 -text 	"Total Cumulative Energy		:"
	label $backup_1.f2.l5 -text 	"Cumulative active Energy T1		:"
	label $backup_1.f2.l6 -text 	"Cumulative active Energy T2		:"
	label $backup_1.f2.l7 -text 	"Cumulative active Energy T3		:"
	label $backup_1.f2.l8 -text 	"Cumulative active Energy T4		:"		
	label $backup_1.f2.l9 -text 	"Cumulative apparent Energy T1	:"
	label $backup_1.f2.l10 -text 	"Cumulative apparent Energy T2	:"
	label $backup_1.f2.l11 -text 	"Cumulative apparent Energy T3	:"
	label $backup_1.f2.l12 -text 	"Cumulative apparent Energy T4	:"			
	foreach i {1 2 3 4 5 6 7 8 9 10 11 12} {pack $backup_1.f2.l$i -side top -expand 0}
	
	label $backup_2.f2.l1 -text 	"Type of Reset			:"              
	label $backup_2.f2.l2 -text 	"Reset Time			:"              
	label $backup_2.f2.l3 -text 	"Reset Date			:"              
	label $backup_2.f2.l4 -text 	"Total Cumulative Energy		:"      
	label $backup_2.f2.l5 -text 	"Cumulative active Energy T1		:"      
	label $backup_2.f2.l6 -text 	"Cumulative active Energy T2		:"      
	label $backup_2.f2.l7 -text 	"Cumulative active Energy T3		:"      
	label $backup_2.f2.l8 -text 	"Cumulative active Energy T4		:"		
	label $backup_2.f2.l9 -text 	"Cumulative apparent Energy T1	:"              
	label $backup_2.f2.l10 -text	"Cumulative apparent Energy T2	:"              
	label $backup_2.f2.l11 -text	"Cumulative apparent Energy T3	:"              
	label $backup_2.f2.l12 -text	"Cumulative apparent Energy T4	:"				
	foreach i {1 2 3 4 5 6 7 8 9 10 11 12} {pack $backup_2.f2.l$i -side top -expand 0 }

	label $backup_3.f2.l1 -text 	"Type of Reset			:"              
	label $backup_3.f2.l2 -text 	"Reset Time			:"              
	label $backup_3.f2.l3 -text 	"Reset Date			:"              
	label $backup_3.f2.l4 -text 	"Total Cumulative Energy		:"      
	label $backup_3.f2.l5 -text 	"Cumulative active Energy T1		:"      
	label $backup_3.f2.l6 -text 	"Cumulative active Energy T2		:"      
	label $backup_3.f2.l7 -text 	"Cumulative active Energy T3		:"      
	label $backup_3.f2.l8 -text 	"Cumulative active Energy T4		:"	
	label $backup_3.f2.l9 -text 	"Cumulative apparent Energy T1	:"              
	label $backup_3.f2.l10 -text	"Cumulative apparent Energy T2	:"              
	label $backup_3.f2.l11 -text	"Cumulative apparent Energy T3	:"              
	label $backup_3.f2.l12 -text	"Cumulative apparent Energy T4	:"		
	foreach i {1 2 3 4 5 6 7 8 9 10 11 12} {pack $backup_3.f2.l$i -side top -expand 0 }

	label $backup_4.f2.l1 -text 	"Type of Reset			:"              
	label $backup_4.f2.l2 -text 	"Reset Time			:"              
	label $backup_4.f2.l3 -text 	"Reset Date			:"              
	label $backup_4.f2.l4 -text 	"Total Cumulative Energy		:"      
	label $backup_4.f2.l5 -text 	"Cumulative active Energy T1		:"      
	label $backup_4.f2.l6 -text 	"Cumulative active Energy T2		:"      
	label $backup_4.f2.l7 -text 	"Cumulative active Energy T3		:"      
	label $backup_4.f2.l8 -text 	"Cumulative active Energy T4		:"	
	label $backup_4.f2.l9 -text 	"Cumulative apparent Energy T1	:"              
	label $backup_4.f2.l10 -text	"Cumulative apparent Energy T2	:"              
	label $backup_4.f2.l11 -text	"Cumulative apparent Energy T3	:"              
	label $backup_4.f2.l12 -text	"Cumulative apparent Energy T4	:"		
	foreach i {1 2 3 4 5 6 7 8 9 10 11 12} {pack $backup_4.f2.l$i -side top -expand 0 }	

	label $backup_5.f2.l1 -text 	"Type of Reset			:"              
	label $backup_5.f2.l2 -text 	"Reset Time			:"              
	label $backup_5.f2.l3 -text 	"Reset Date			:"              
	label $backup_5.f2.l4 -text 	"Total Cumulative Energy		:"      
	label $backup_5.f2.l5 -text 	"Cumulative active Energy T1		:"      
	label $backup_5.f2.l6 -text 	"Cumulative active Energy T2		:"      
	label $backup_5.f2.l7 -text 	"Cumulative active Energy T3		:"      
	label $backup_5.f2.l8 -text 	"Cumulative active Energy T4		:"	
	label $backup_5.f2.l9 -text 	"Cumulative apparent Energy T1	:"              
	label $backup_5.f2.l10 -text	"Cumulative apparent Energy T2	:"              
	label $backup_5.f2.l11 -text	"Cumulative apparent Energy T3	:"              
	label $backup_5.f2.l12 -text	"Cumulative apparent Energy T4	:"		
	foreach i {1 2 3 4 5 6 7 8 9 10 11 12} {pack $backup_5.f2.l$i -side top -expand 0 }     
                                                                                                
	label $backup_6.f2.l1 -text 	"Type of Reset			:"                
	label $backup_6.f2.l2 -text 	"Reset Time			:"                
	label $backup_6.f2.l3 -text 	"Reset Date			:"                
	label $backup_6.f2.l4 -text 	"Total Cumulative Energy		:"        
	label $backup_6.f2.l5 -text 	"Cumulative active Energy T1		:"        
	label $backup_6.f2.l6 -text 	"Cumulative active Energy T2		:"        
	label $backup_6.f2.l7 -text 	"Cumulative active Energy T3		:"        
	label $backup_6.f2.l8 -text 	"Cumulative active Energy T4		:"	  
	label $backup_6.f2.l9 -text 	"Cumulative apparent Energy T1	:"                
	label $backup_6.f2.l10 -text	"Cumulative apparent Energy T2	:"                
	label $backup_6.f2.l11 -text	"Cumulative apparent Energy T3	:"                
	label $backup_6.f2.l12 -text	"Cumulative apparent Energy T4	:"		  
	foreach i {1 2 3 4 5 6 7 8 9 10 11 12} {pack $backup_6.f2.l$i -side top -expand 0 }	
	
	frame $backup_1.f1
	frame $backup_2.f1
	frame $backup_3.f1
	frame $backup_4.f1
	frame $backup_5.f1
	frame $backup_6.f1
	pack $backup_1.f1 -fill both -side left 
	pack $backup_2.f1 -fill both -side left 
	pack $backup_3.f1 -fill both -side left 
	pack $backup_4.f1 -fill both -side left 
	pack $backup_5.f1 -fill both -side left 
	pack $backup_6.f1 -fill both -side left 
	
	set b1_rst_type ""
	set b2_rst_type ""
	set b3_rst_type ""
	set b4_rst_type ""
	set b5_rst_type ""
	set b6_rst_type ""
	label $backup_1.f1.l1 -text "$b1_rst_type		" 
	label $backup_2.f1.l1 -text "$b2_rst_type		"  	
	label $backup_3.f1.l1 -text "$b3_rst_type		"  
	label $backup_4.f1.l1 -text "$b4_rst_type		"  
	label $backup_5.f1.l1 -text "$b5_rst_type		"  
	label $backup_6.f1.l1 -text "$b6_rst_type		"  
	pack $backup_1.f1.l1 -ipadx 0 -ipady 0
	pack $backup_2.f1.l1 -ipadx 0 -ipady 0
	pack $backup_3.f1.l1 -ipadx 0 -ipady 0
	pack $backup_4.f1.l1 -ipadx 0 -ipady 0
	pack $backup_5.f1.l1 -ipadx 0 -ipady 0
	pack $backup_6.f1.l1 -ipadx 0 -ipady 0
                                    
	set b1_rst_time ""
	set b2_rst_time ""
	set b3_rst_time ""
	set b4_rst_time ""
	set b5_rst_time ""
	set b6_rst_time ""
	
	label $backup_1.f1.l2 -text "$b1_rst_time			" 
	label $backup_2.f1.l2 -text "$b2_rst_time			" 
	label $backup_3.f1.l2 -text "$b3_rst_time			" 
	label $backup_4.f1.l2 -text "$b4_rst_time			"  
	label $backup_5.f1.l2 -text "$b5_rst_time			" 
	label $backup_6.f1.l2 -text "$b6_rst_time			"  
	pack $backup_1.f1.l2 -ipadx 0 -ipady 0
	pack $backup_2.f1.l2 -ipadx 0 -ipady 0
	pack $backup_3.f1.l2 -ipadx 0 -ipady 0
	pack $backup_4.f1.l2 -ipadx 0 -ipady 0
	pack $backup_5.f1.l2 -ipadx 0 -ipady 0
	pack $backup_6.f1.l2 -ipadx 0 -ipady 0	

	set b1_rst_date ""                                                                         
	set b2_rst_date ""                                                                         
	set b3_rst_date ""                                                                         
	set b4_rst_date ""                                                                         
	set b5_rst_date ""                                                                         
	set b6_rst_date ""                                                                         
	                                                                
	label 	$backup_1.f1.l3 -text "$b1_rst_date		"  
	label 	$backup_2.f1.l3 -text "$b2_rst_date		"       
	label 	$backup_3.f1.l3 -text "$b3_rst_date		"  
	label 	$backup_4.f1.l3 -text "$b4_rst_date		"       
	label 	$backup_5.f1.l3 -text "$b5_rst_date		"  
	label 	$backup_6.f1.l3 -text "$b6_rst_date		"       
	pack  	$backup_1.f1.l3 -ipadx 0 -ipady 0                                                         
	pack  	$backup_2.f1.l3 -ipadx 0 -ipady 0                                                         
	pack  	$backup_3.f1.l3 -ipadx 0 -ipady 0                                                         
	pack  	$backup_4.f1.l3 -ipadx 0 -ipady 0                                                         
	pack  	$backup_5.f1.l3 -ipadx 0 -ipady 0                                                         
	pack  	$backup_6.f1.l3 -ipadx 0 -ipady 0	                                                       

	set b1_total_energy ""                                                                         
	set b2_total_energy ""                                                                         
	set b3_total_energy ""                                                                         
	set b4_total_energy ""                                                                         
	set b5_total_energy ""                                                                         
	set b6_total_energy ""                                                                         
	                                                                                         
	label 	$backup_1.f1.l4 -text "$b1_total_energy			"  
	label 	$backup_2.f1.l4 -text "$b2_total_energy			"         
	label 	$backup_3.f1.l4 -text "$b3_total_energy			"  
	label 	$backup_4.f1.l4 -text "$b4_total_energy			"         
	label 	$backup_5.f1.l4 -text "$b5_total_energy			"  
	label 	$backup_6.f1.l4 -text "$b6_total_energy			"         
	pack  	$backup_1.f1.l4 -ipadx 0 -ipady 0                            
	pack  	$backup_2.f1.l4 -ipadx 0 -ipady 0                                                         
	pack  	$backup_3.f1.l4 -ipadx 0 -ipady 0                                                         
	pack  	$backup_4.f1.l4 -ipadx 0 -ipady 0                                                         
	pack  	$backup_5.f1.l4 -ipadx 0 -ipady 0                                                         
	pack  	$backup_6.f1.l4 -ipadx 0 -ipady 0	                                      
	
	set b1_act_energy_t1 ""                                                                         
	set b2_act_energy_t1 ""                                                                         
	set b3_act_energy_t1 ""                                                                         
	set b4_act_energy_t1 ""                                                                         
	set b5_act_energy_t1 ""                                                                         
	set b6_act_energy_t1 ""                                                                         
	                                                                                               
	label 	$backup_1.f1.l5 -text "$b1_act_energy_t1			"  
	label 	$backup_2.f1.l5 -text "$b2_act_energy_t1			"         
	label 	$backup_3.f1.l5 -text "$b3_act_energy_t1			"  
	label 	$backup_4.f1.l5 -text "$b4_act_energy_t1			"         
	label 	$backup_5.f1.l5 -text "$b5_act_energy_t1			"  
	label 	$backup_6.f1.l5 -text "$b6_act_energy_t1			"         
	pack  	$backup_1.f1.l5 -ipadx 0 -ipady 0                            
	pack  	$backup_2.f1.l5 -ipadx 0 -ipady 0                                                         
	pack  	$backup_3.f1.l5 -ipadx 0 -ipady 0                                                         
	pack  	$backup_4.f1.l5 -ipadx 0 -ipady 0                                                         
	pack  	$backup_5.f1.l5 -ipadx 0 -ipady 0                                                         
	pack  	$backup_6.f1.l5 -ipadx 0 -ipady 0	                 	

	set b1_act_energy_t2 ""                                            
	set b2_act_energy_t2 ""                                            
	set b3_act_energy_t2 ""                                            
	set b4_act_energy_t2 ""                                            
	set b5_act_energy_t2 ""                                            
	set b6_act_energy_t2 ""                                            
	                                                                         
	label 	$backup_1.f1.l6 -text "$b1_act_energy_t2			"
	label 	$backup_2.f1.l6 -text "$b2_act_energy_t2			"
	label 	$backup_3.f1.l6 -text "$b3_act_energy_t2			"
	label 	$backup_4.f1.l6 -text "$b4_act_energy_t2			"
	label 	$backup_5.f1.l6 -text "$b5_act_energy_t2			"
	label 	$backup_6.f1.l6 -text "$b6_act_energy_t2			"
	pack  	$backup_1.f1.l6 -ipadx 0 -ipady 0                                
	pack  	$backup_2.f1.l6 -ipadx 0 -ipady 0                                
	pack  	$backup_3.f1.l6 -ipadx 0 -ipady 0                                
	pack  	$backup_4.f1.l6 -ipadx 0 -ipady 0                                
	pack  	$backup_5.f1.l6 -ipadx 0 -ipady 0                                
	pack  	$backup_6.f1.l6 -ipadx 0 -ipady 0	                 	 
	
	set b1_act_energy_t3 ""                                              
	set b2_act_energy_t3 ""                                              
	set b3_act_energy_t3 ""                                              
	set b4_act_energy_t3 ""                                              
	set b5_act_energy_t3 ""                                              
	set b6_act_energy_t3 ""                                              
	                                                                           
	label 	$backup_1.f1.l7 -text "$b1_act_energy_t3			"  
	label 	$backup_2.f1.l7 -text "$b2_act_energy_t3			"  
	label 	$backup_3.f1.l7 -text "$b3_act_energy_t3			"  
	label 	$backup_4.f1.l7 -text "$b4_act_energy_t3			"  
	label 	$backup_5.f1.l7 -text "$b5_act_energy_t3			"  
	label 	$backup_6.f1.l7 -text "$b6_act_energy_t3			"  
	pack  	$backup_1.f1.l7 -ipadx 0 -ipady 0                                  
	pack  	$backup_2.f1.l7 -ipadx 0 -ipady 0                                  
	pack  	$backup_3.f1.l7 -ipadx 0 -ipady 0                                  
	pack  	$backup_4.f1.l7 -ipadx 0 -ipady 0                                  
	pack  	$backup_5.f1.l7 -ipadx 0 -ipady 0                                  
	pack  	$backup_6.f1.l7 -ipadx 0 -ipady 0	                 	   

	set b1_act_energy_t4 ""                                                                         
	set b2_act_energy_t4 ""                                                                         
	set b3_act_energy_t4 ""                                                                         
	set b4_act_energy_t4 ""                                                                         
	set b5_act_energy_t4 ""                                                                         
	set b6_act_energy_t4 ""                                                                         
	                                                                                               
	label 	$backup_1.f1.l8 -text "$b1_act_energy_t4			"  
	label 	$backup_2.f1.l8 -text "$b2_act_energy_t4			"         
	label 	$backup_3.f1.l8 -text "$b3_act_energy_t4			"  
	label 	$backup_4.f1.l8 -text "$b4_act_energy_t4			"         
	label 	$backup_5.f1.l8 -text "$b5_act_energy_t4			"  
	label 	$backup_6.f1.l8 -text "$b6_act_energy_t4			"         
	pack  	$backup_1.f1.l8 -ipadx 0 -ipady 0                            
	pack  	$backup_2.f1.l8 -ipadx 0 -ipady 0                                                         
	pack  	$backup_3.f1.l8 -ipadx 0 -ipady 0                                                         
	pack  	$backup_4.f1.l8 -ipadx 0 -ipady 0                                                         
	pack  	$backup_5.f1.l8 -ipadx 0 -ipady 0                                                         
	pack  	$backup_6.f1.l8 -ipadx 0 -ipady 0	                 	

	set b1_app_energy_t1 ""                                            
	set b2_app_energy_t1 ""                                            
	set b3_app_energy_t1 ""                                            
	set b4_app_energy_t1 ""                                            
	set b5_app_energy_t1 ""                                            
	set b6_app_energy_t1 ""                                            
	                                                                         
	label 	$backup_1.f1.l9 -text "$b1_app_energy_t1			"
	label 	$backup_2.f1.l9 -text "$b2_app_energy_t1			"
	label 	$backup_3.f1.l9 -text "$b3_app_energy_t1			"
	label 	$backup_4.f1.l9 -text "$b4_app_energy_t1			"
	label 	$backup_5.f1.l9 -text "$b5_app_energy_t1			"
	label 	$backup_6.f1.l9 -text "$b6_app_energy_t1			"
	pack  	$backup_1.f1.l9 -ipadx 0 -ipady 0                                
	pack  	$backup_2.f1.l9 -ipadx 0 -ipady 0                                
	pack  	$backup_3.f1.l9 -ipadx 0 -ipady 0                                
	pack  	$backup_4.f1.l9 -ipadx 0 -ipady 0                                
	pack  	$backup_5.f1.l9 -ipadx 0 -ipady 0                                
	pack  	$backup_6.f1.l9 -ipadx 0 -ipady 0	                 	 
	
	set b1_app_energy_t2 ""                                            
	set b2_app_energy_t2 ""                                            
	set b3_app_energy_t2 ""                                            
	set b4_app_energy_t2 ""                                            
	set b5_app_energy_t2 ""                                            
	set b6_app_energy_t2 ""                                            
	                                                                         
	label 	$backup_1.f1.l10 -text "$b1_app_energy_t2			"
	label 	$backup_2.f1.l10 -text "$b2_app_energy_t2			"
	label 	$backup_3.f1.l10 -text "$b3_app_energy_t2			"
	label 	$backup_4.f1.l10 -text "$b4_app_energy_t2			"
	label 	$backup_5.f1.l10 -text "$b5_app_energy_t2			"
	label 	$backup_6.f1.l10 -text "$b6_app_energy_t2			"
	pack  	$backup_1.f1.l10 -ipadx 0 -ipady 0                                
	pack  	$backup_2.f1.l10 -ipadx 0 -ipady 0                                
	pack  	$backup_3.f1.l10 -ipadx 0 -ipady 0                                
	pack  	$backup_4.f1.l10 -ipadx 0 -ipady 0                                
	pack  	$backup_5.f1.l10 -ipadx 0 -ipady 0                                
	pack  	$backup_6.f1.l10 -ipadx 0 -ipady 0	               

	set b1_app_energy_t3 ""                                            
	set b2_app_energy_t3 ""                                            
	set b3_app_energy_t3 ""                                            
	set b4_app_energy_t3 ""                                            
	set b5_app_energy_t3 ""                                            
	set b6_app_energy_t3 ""                                            
	                                                                         
	label 	$backup_1.f1.l11 -text "$b1_app_energy_t3			"
	label 	$backup_2.f1.l11 -text "$b2_app_energy_t3			"
	label 	$backup_3.f1.l11 -text "$b3_app_energy_t3			"
	label 	$backup_4.f1.l11 -text "$b4_app_energy_t3			"
	label 	$backup_5.f1.l11 -text "$b5_app_energy_t3			"
	label 	$backup_6.f1.l11 -text "$b6_app_energy_t3			"
	pack  	$backup_1.f1.l11 -ipadx 0 -ipady 0                                
	pack  	$backup_2.f1.l11 -ipadx 0 -ipady 0                                
	pack  	$backup_3.f1.l11 -ipadx 0 -ipady 0                                
	pack  	$backup_4.f1.l11 -ipadx 0 -ipady 0                                
	pack  	$backup_5.f1.l11 -ipadx 0 -ipady 0                                
	pack  	$backup_6.f1.l11 -ipadx 0 -ipady 0	               
	
	set b1_app_energy_t4 ""                                            
	set b2_app_energy_t4 ""                                            
	set b3_app_energy_t4 ""                                            
	set b4_app_energy_t4 ""                                            
	set b5_app_energy_t4 ""                                            
	set b6_app_energy_t4 ""                                            
	                                                                         
	label 	$backup_1.f1.l12 -text "$b1_app_energy_t4			"
	label 	$backup_2.f1.l12 -text "$b2_app_energy_t4			"
	label 	$backup_3.f1.l12 -text "$b3_app_energy_t4			"
	label 	$backup_4.f1.l12 -text "$b4_app_energy_t4			"
	label 	$backup_5.f1.l12 -text "$b5_app_energy_t4			"
	label 	$backup_6.f1.l12 -text "$b6_app_energy_t4			"
	pack  	$backup_1.f1.l12 -ipadx 0 -ipady 0                                
	pack  	$backup_2.f1.l12 -ipadx 0 -ipady 0                                
	pack  	$backup_3.f1.l12 -ipadx 0 -ipady 0                                
	pack  	$backup_4.f1.l12 -ipadx 0 -ipady 0                                
	pack  	$backup_5.f1.l12 -ipadx 0 -ipady 0                                
	pack  	$backup_6.f1.l12 -ipadx 0 -ipady 0	     
	
         	

#########################################################
#
# Programming Notebook
#########################################################
	frame $v.title 
	pack $v.title -side left -anchor ne
	
	frame $v.title.ftop
	pack $v.title.ftop -side top -anchor ne

	set 	scroll_period [labelframe $v.title.ftop.lf1 -text "Scroll Period" -font *-ansi-bold-r-*-12-*] 
	set 	unit_code [labelframe $v.title.ftop.lf2 -text "Unit Code" -font *-ansi-bold-r-*-12-*] 
	set 	int_period [labelframe $v.title.ftop.lf3 -text "Integration Period" -font *-ansi-bold-r-*-12-*] 
	pack 	$scroll_period $unit_code $int_period -side left -expand 1 -anchor n

	frame $v.title.fmid
	pack $v.title.fmid -side top -anchor w 
	
	frame $v.title.fbottom
	pack $v.title.fbottom -side top -anchor sw 
	
	frame $v.title.fupdate_ram
	pack $v.title.fupdate_ram -side top -fill both -anchor center -anchor s
	
	
	set 	tariff_setting [labelframe $v.title.fmid.lf4 -text "Tariff Setting" -font *-ansi-bold-r-*-12-*] 
	pack	$tariff_setting -side top -anchor n

	set 	rtc_setting [labelframe $v.title.fbottom.lf4 -text "RTC Setting" -font *-ansi-bold-r-*-12-*] 
	pack	$rtc_setting -side top -anchor s
	
###################################
# Scroll Period
###################################
	set 	scroll_period1 [frame $scroll_period.f1]
	set 	scroll_period2 [frame $scroll_period.f2]
	pack 	$scroll_period1 $scroll_period2 -side left -fill both -expand 1 -padx 3 -pady 3
	
	label 	$scroll_period1.l1 -text "Present Value:"
	label 	$scroll_period1.l2 -text "Desire Value:"
	foreach i {1 2} {pack $scroll_period1.l$i -side top  -anchor nw}	
	button 	$scroll_period.b1 -text "CHANGE" -command {
					if {$desire_scroll_period==10||  \
					$desire_scroll_period==20||  \
					$desire_scroll_period==30} {  \
						# update to the meter
						reinitialize_con
						switch_program_mode																				
						write_scroll_period $desire_scroll_period										
						update_ram
					 	$scroll_period2.l1 configure -text $desire_scroll_period 					 						 	
					 	} \
					else {tk_messageBox -title Input_Invalid -message "Please input 10, 20 and 30 only!" -icon warning}
					}

	pack 	$scroll_period.b1 -side left -fill y
	#read the scroll period from the meter
	
	
		
	


	set 	present_scroll_period ""
		
	label 	$scroll_period2.l1 -text $present_scroll_period	
	entry 	$scroll_period2.e1 -relief sunken -state normal  -textvariable desire_scroll_period -width 5
	pack 	$scroll_period2.l1 -side top  -anchor nw
	pack 	$scroll_period2.e1 -side top 

###################################
# Unit Code
###################################	
	set 	unit_code1 [frame $unit_code.f1]
	set 	unit_code2 [frame $unit_code.f2]
	pack 	$unit_code1 $unit_code2 -side left -fill both -expand 1 -padx 3 -pady 3
	
	label 	$unit_code1.l1 -text "Present Value:"
	label 	$unit_code1.l2 -text "Desire Value:"
	foreach i {1 2} {pack $unit_code1.l$i -side top  -anchor nw}	
	
	button 	$unit_code.b1 -text "CHANGE" -command {
					if {	[string length $desire_unit_code]==6 &&\
						[regexp {[^A-z0-9]} $desire_unit_code]==0
						} {  
						# update to the meter
						reinitialize_con
						switch_program_mode																				
						write_unit_code $desire_unit_code									
						update_ram
					 	$unit_code2.l1 configure -text $desire_unit_code 					 						 	
					 	} \
					else {tk_messageBox -title Input_Invalid -message "Please input only 6 alphanumerics" -icon warning}

					
					}

	pack 	$unit_code.b1 -side left -fill y
	
	#read in unit_code from the meter
	set 	present_unit_code ""
	
	label 	$unit_code2.l1 -text $present_unit_code	
	entry 	$unit_code2.e1 -relief sunken -state normal  -textvariable desire_unit_code -width 6
	pack 	$unit_code2.l1 -side top  -anchor nw
	pack 	$unit_code2.e1 -side top 
	
###################################
# integration period
###################################	
	set 	int_period1 [frame $int_period.f1]
	set 	int_period2 [frame $int_period.f2]
	pack 	$int_period1 $int_period2 -side left -fill both -expand 1 -padx 3 -pady 3
	
	label 	$int_period1.l1 -text "Present Value:"
	label 	$int_period1.l2 -text "Desire Value:"
	foreach i {1 2} {pack $int_period1.l$i -side top  -anchor nw}	
	
	button 	$int_period.b1 -text "CHANGE" -command {
					if {$desire_int_period==30||  \
					$desire_int_period==60} {  \
						# update to the meter
						reinitialize_con
						switch_program_mode																				
						write_md_period $desire_int_period										
						update_ram
					 	$int_period2.l1 configure -text $desire_int_period 					 						 	
					 	} \
					else {tk_messageBox -title Input_Invalid -message "Please input 30 and 60 only!" -icon warning}

					}

	pack 	$int_period.b1 -side left -fill y
	set 	present_int_period ""
	label 	$int_period2.l1 -text $present_int_period	
	entry 	$int_period2.e1 -relief sunken -state normal  -textvariable desire_int_period -width 5
	pack 	$int_period2.l1 -side top  -anchor nw
	pack 	$int_period2.e1 -side top                                        
	
###################################
# Tariff Setting
###################################	
	set 	tariff_set1 [frame $tariff_setting.f1]
	set 	tariff_set2 [frame $tariff_setting.f2]
	set 	tariff_set3 [frame $tariff_setting.f3]
	set 	tariff_set4 [frame $tariff_setting.f4]
	set 	tariff_set5 [frame $tariff_setting.f5]
	set 	tariff_set6 [frame $tariff_setting.f6]
	
	for {set i 1} {$i<7} {incr i} {	pack 	$tariff_setting.f$i -side left -fill both -expand 1 -padx 3 -pady 3}
	
	label 	$tariff_set1.l1	-text "Slot No.\t" -font {times 12 {bold italic underline}}
	label 	$tariff_set1.l2	-text "1." 
	label 	$tariff_set1.l3	-text "2." 
	label 	$tariff_set1.l4	-text "3." 
	label 	$tariff_set1.l5	-text "4." 
	label 	$tariff_set1.l6	-text "5." 
	label 	$tariff_set1.l7	-text "6." 
	for {set i 1} {$i<8} {incr i} {pack $tariff_set1.l$i -side top} 
	
	
	set t_slot1 	""
	set t_slot2 	""
	set t_slot3 	""
	set t_slot4 	""
	set t_slot5 	""
	set t_slot6 	""
                        
	set st_slot1 	""
	set st_slot2 	""
	set st_slot3 	""
	set st_slot4 	""
	set st_slot5 	""
	set st_slot6 	""
                        
	set et_slot1 	""
	set et_slot2 	""
	set et_slot3 	""
	set et_slot4 	""
	set et_slot5    ""
	set et_slot6 	""
	

	
	label 	$tariff_set2.l1	-text "Tariff data\t" -font {times 12 {bold italic underline}}	
	entry 	$tariff_set2.e2	-textvariable t_slot1 -width 6   
	entry 	$tariff_set2.e3	-textvariable t_slot2 -width 6    
	entry 	$tariff_set2.e4	-textvariable t_slot3 -width 6    
	entry 	$tariff_set2.e5	-textvariable t_slot4 -width 6    
	entry 	$tariff_set2.e6	-textvariable t_slot5 -width 6    
	entry 	$tariff_set2.e7	-textvariable t_slot6 -width 6    
	pack $tariff_set2.l1 -side top	
	for {set i 2} {$i<8} {incr i} {pack $tariff_set2.e$i -side top}

	label 	$tariff_set3.l1	-text "Start Time\t" -font {times 12 {bold italic underline}}		
	entry 	$tariff_set3.e2	-textvariable st_slot1 -width 6   
	entry 	$tariff_set3.e3	-textvariable st_slot2 -width 6    
	entry 	$tariff_set3.e4	-textvariable st_slot3 -width 6    
	entry 	$tariff_set3.e5	-textvariable st_slot4 -width 6    
	entry 	$tariff_set3.e6	-textvariable st_slot5 -width 6    
	entry 	$tariff_set3.e7	-textvariable st_slot6 -width 6    
	pack $tariff_set3.l1 -side top	
	for {set i 2} {$i<8} {incr i} {pack $tariff_set3.e$i -side top}

	label 	$tariff_set4.l1	-text "End Time\t" -font {times 12 {bold italic underline}}
	entry 	$tariff_set4.e2	-textvariable et_slot1 -width 6   
	entry 	$tariff_set4.e3	-textvariable et_slot2 -width 6    
	entry 	$tariff_set4.e4	-textvariable et_slot3 -width 6    
	entry 	$tariff_set4.e5	-textvariable et_slot4 -width 6    
	entry 	$tariff_set4.e6	-textvariable et_slot5 -width 6    
	entry 	$tariff_set4.e7	-textvariable et_slot6 -width 6    
	pack $tariff_set4.l1 -side top	
	for {set i 2} {$i<8} {incr i} {pack $tariff_set4.e$i -side top}

	button 	$tariff_set5.b1 -text "CHANGE" -command {
						# check for valid start time and end time
						
						
						if {!([check_valid_time  $st_slot1] && [check_valid_time  $et_slot1] &&\
							[check_valid_time  $st_slot2] && [check_valid_time  $et_slot2] &&\
	                 			   	[check_valid_time  $st_slot3] && [check_valid_time  $et_slot3] &&\
						        [check_valid_time  $st_slot4] && [check_valid_time  $et_slot4] &&\
						        [check_valid_time  $st_slot5] && [check_valid_time  $et_slot5] &&\
						        [check_valid_time  $st_slot6] && [check_valid_time  $et_slot6])} {
							
							tk_messageBox -title Input_Invalid -message "Invalid Tariff time" -icon warning	
						} else { # write tariffs
							set tariffs 	"$t_slot1 \
								 $t_slot2 \
								 $t_slot3 \
								 $t_slot4 \
								 $t_slot5 \
								 $t_slot6 \
								 [lindex [regsub {:} $st_slot1 { }] 1] \
								 [lindex [regsub {:} $st_slot1 { }] 0] \
								 [lindex [regsub {:} $et_slot1 { }] 1] \
								 [lindex [regsub {:} $et_slot1 { }] 0] \
								 [lindex [regsub {:} $st_slot2 { }] 1] \
								 [lindex [regsub {:} $st_slot2 { }] 0] \
								 [lindex [regsub {:} $et_slot2 { }] 1] \
								 [lindex [regsub {:} $et_slot2 { }] 0] \
								 [lindex [regsub {:} $st_slot3 { }] 1] \
								 [lindex [regsub {:} $st_slot3 { }] 0] \
								 [lindex [regsub {:} $et_slot3 { }] 1] \
								 [lindex [regsub {:} $et_slot3 { }] 0] \
								 [lindex [regsub {:} $st_slot4 { }] 1] \
								 [lindex [regsub {:} $st_slot4 { }] 0] \
								 [lindex [regsub {:} $et_slot4 { }] 1] \
								 [lindex [regsub {:} $et_slot4 { }] 0] \
								 [lindex [regsub {:} $st_slot5 { }] 1] \
								 [lindex [regsub {:} $st_slot5 { }] 0] \
								 [lindex [regsub {:} $et_slot5 { }] 1] \
								 [lindex [regsub {:} $et_slot5 { }] 0] \
								 [lindex [regsub {:} $st_slot6 { }] 1] \
								 [lindex [regsub {:} $st_slot6 { }] 0] \
								 [lindex [regsub {:} $et_slot6 { }] 1] \
								 [lindex [regsub {:} $et_slot6 { }] 0]"
								 
							 	write_tariffs $tariffs  
							 	update_ram						 						  
							 }
						
					}					     																   
				                                         

	pack 	$tariff_set5.b1 -side left -fill y
	
###################################
# RTC Setting
###################################	
	set 	rtc_set1 [frame $rtc_setting.f1]
	set 	rtc_set2 [frame $rtc_setting.f2]
	set 	rtc_set3 [frame $rtc_setting.f3]
	set 	rtc_set4 [frame $rtc_setting.f4]
	#for 	{set i 1} {$i<3} {incr i} {pack $rtc_setting.f$i -side left -fill both -expand 1 -padx 3 -pady 3}
	pack $rtc_setting.f1 -side left -padx 3 -pady 3 -anchor nw
	pack $rtc_setting.f2 -side top -fill both -expand 1 -padx 3 -pady 3
	pack $rtc_setting.f3 -side left -fill both -expand 1 -padx 3 -pady 3 -anchor w
	pack $rtc_setting.f4 -side left -fill both -expand 1 -padx 3 -pady 3 -anchor w
	
	frame 	$rtc_set2.upframe
	frame 	$rtc_set2.downframe
	pack 	$rtc_set2.upframe -side top -anchor nw
	pack 	$rtc_set2.downframe -side top 

	set 	rtc_set2up $rtc_setting.f2.upframe
	set 	rtc_set2down $rtc_setting.f2.downframe
	
	label 	$rtc_setting.f1.l1	-text "Time:\t" -font {times 12 {bold}}
	label 	$rtc_setting.f1.l2	-text "Date:\t" -font {times 12 {bold}}	
	#label 	$rtc_setting.f1.l3	-text "(dd-mm-yyyy)\t" 
	label 	$rtc_setting.f1.l3	-text "Day of Week:\t" 
	for {set i 1} {$i<4} {incr i} {pack $rtc_setting.f1.l$i -side top -anchor nw}	

	
	set rtc_hh 	  ""
	set rtc_mm 	  ""
	set rtc_ss 	  ""
	set rtc_dd 	  ""
	set rtc_mon 	  ""
	set rtc_yy 	  ""
	set rtc_day_prog  ""
	
	entry 	$rtc_set2up.e1	-textvariable  rtc_hh -width 2 
	label   $rtc_set2up.e2	-text ":" 
	entry 	$rtc_set2up.e3	-textvariable  rtc_mm -width 2    
	label   $rtc_set2up.e4	-text ":" 
	entry 	$rtc_set2up.e5	-textvariable  rtc_ss -width 2   
	label 	$rtc_set2up.e6	-text "(hh:mm:ss)\t"
	for {set i 1} {$i<7} {incr i} {pack $rtc_set2up.e$i 	-side left -padx 3 -pady 3 -anchor nw}

	entry 	$rtc_set2down.e4	-textvariable rtc_dd -width 2  
	label   $rtc_set2down.e5	-text "-" 
	entry 	$rtc_set2down.e6	-textvariable rtc_mon -width 2    
	label   $rtc_set2down.e7	-text "-" 	 
	entry 	$rtc_set2down.e8	-textvariable rtc_yy -width 4   
	label 	$rtc_set2down.e9	-text "(dd-mm-yyyy)\t"
	for {set i 4} {$i<10} {incr i} {pack $rtc_set2down.e$i -side left -fill both -expand 1 -padx 3 -pady 0 -anchor w}	
	 
	
	entry 	$rtc_set3.e1	-textvariable rtc_day_prog
	pack 	$rtc_set3.e1 	-side left -anchor n
	
	button 	$rtc_set4.b1 -text "CHANGE" -command {
						# check for valid time & date 												
							set ok 1
							if {[check_valid_time ${rtc_hh}:${rtc_mm}:${rtc_ss}]==0} {
								tk_messageBox -title Input_Invalid -message "Invalid RTC time" -icon warning
								set ok 0
							}						
							if {[check_valid_date ${rtc_dd}/${rtc_mon}/${rtc_yy}]==0} {
								tk_messageBox -title Input_Invalid -message "Invalid RTC date" -icon warning
								set ok 0
							}						
														
							if { $rtc_day_prog < 0  ||  $rtc_day_prog > 6} {
								tk_messageBox -title Input_Invalid -message "Invalid RTC day of week" -icon warning
								set ok 0
							}										

							
							if {$ok} {								
								# note : should put 0 in front of less-than-10 numbers 								
								if {[string length $rtc_ss	 ] < 2} { set rtc_ss  0${rtc_ss}}	
								if {[string length $rtc_mm	 ] < 2} { set rtc_mm  0${rtc_mm}}	
								if {[string length $rtc_hh	 ] < 2} { set rtc_hh  0${rtc_hh}}	
								if {[string length $rtc_dd	 ] < 2} { set rtc_dd  0${rtc_dd}}	
								if {[string length $$rtc_day_prog	 ] < 2} { set $rtc_day_prog  0${$rtc_day_prog}}	
								if {[string length $rtc_mon	 ] < 2} { set rtc_mon 0${rtc_mon}}	
								write_rtc "$rtc_ss $rtc_mm $rtc_hh $rtc_dd  $rtc_day_prog $rtc_mon 0[expr $rtc_yy-2000]"
																
							}
							
						}

	pack 	$rtc_set4.b1 -side left -fill both -expand 1

	
	
#########################################################
#
# Calibration Notebook
#########################################################

	frame $x.title 
	pack $x.title -side left -anchor ne
	
	frame $x.title.ftop
	pack $x.title.ftop -side top -anchor ne
	
	Notebook:create $x.title.ftop.n -pages {"Voltage Calibration" "Gain Calibration" "Phase Angle Calibration" "All coefficients" "Present Readings"} -pad 20
	pack $x.title.ftop.n -fill both -ipadx 100 -ipady 100
	
	set 	voltage_cal 	 [labelframe $x.title.ftop.n.f0.lf1 -text "Voltage Calibration" -font *-ansi-bold-r-*-12-*] 
	set 	gain_cal 	 [labelframe $x.title.ftop.n.f1.lf2 -text "Gain Calibration" -font *-ansi-bold-r-*-12-*] 
	set 	phase_angle_cal  [labelframe $x.title.ftop.n.f2.lf3 -text "Phase Angle Calibration" -font *-ansi-bold-r-*-12-*] 
	set 	all_coefficients [labelframe $x.title.ftop.n.f3.lf4 -text "All Coefficients" -font *-ansi-bold-r-*-12-*] 
	set 	Present_Reading  [labelframe $x.title.ftop.n.f4.lf5 -text "Present Readings" -font *-ansi-bold-r-*-12-*]

	pack $x.title.ftop.n.f0.lf1 -side top -anchor nw
	pack $x.title.ftop.n.f1.lf2 -side top -anchor nw
	pack $x.title.ftop.n.f2.lf3 -side top -anchor nw		
	pack $x.title.ftop.n.f3.lf4 -side top -anchor nw		
	pack $x.title.ftop.n.f4.lf5 -side top -anchor nw

# Voltage Calibration Frame
#########################################################

	set reference_std_voltage 240 
	set present_voltage 244
	#set voltage_coefficient [format "%0.4f" [expr $reference_std_voltage/$present_voltage]]
	
	frame 	$voltage_cal.f1
	frame 	$voltage_cal.f2
	pack 	$voltage_cal.f1 -side left -anchor nw
	pack 	$voltage_cal.f2 -side left -anchor nw
	
	
	
	label   $voltage_cal.f1.l3	-text "Calculated Voltage coefficient, Y/X : "
	entry 	$voltage_cal.f2.l3      -textvariable voltage_coefficient -width 6

	
	

	button 	$voltage_cal.f1.l4	-text "Calibrate" -command { \
					# calibrate the meter
					exit_all_modes
					reinitialize_con
					switch_calibration_mode
					
					set t [read_voltage_coefficient1]
					
					set coeff [expr $voltage_coefficient/[lindex $t 3]]
					
					calibrate_voltage_coefficient $coeff 
			}	
	
			
	for {set i 3} {$i<5} {incr i} {pack $voltage_cal.f1.l$i -side top -anchor nw}	

	for {set i 3} {$i<4} {incr i} {pack $voltage_cal.f2.l$i -side top -anchor nw}	
	
			
			
	
 
# Gain Calibration Frame
#########################################################
	 
	
	frame 	$gain_cal.f1
	frame 	$gain_cal.f2
	pack 	$gain_cal.f1 -side left -anchor nw
	pack 	$gain_cal.f2 -side left -anchor nw
	
	#label   $gain_cal.f1.l1	-text "Phase Current: " -font {times 10 {bold italic underline}}
	
	label   $gain_cal.f1.l5	-text "Calculated Gain coefficient: "
	entry 	$gain_cal.f2.l4 -textvariable gain_coefficient -width 6
	
	

	button 	$gain_cal.f1.l6	-text "Calibrate" -command { \
					#puts $gain_number
					
					exit_all_modes
					reinitialize_con
					switch_calibration_mode
										
					set coeff1 [expr $gain_coefficient/[lindex $coes [expr $gain_number-1]]]
										
					calibrate_gain_coefficient C$gain_number $coeff1
					
				
			}	
			
	
	for {set i 5} {$i<7} {incr i} {pack $gain_cal.f1.l$i -side top -anchor nw}	
	#label   $gain_cal.f2.l1		
	
	
	
	for {set i 4} {$i<5} {incr i} {pack $gain_cal.f2.l$i -side top -anchor nw}			
	
	foreach {p q } {
	"1" "Gain 1 for Phase current"
	"2" "Gain 2 for Phase current"
	"3" "Gain 3 for Phase current"
	"4" "Gain 4 for Phase current"
	"5" "Gain 1 for Neutral current"
	"6" "Gain 2 for Neutral current"
	"7" "Gain 3 for Neutral current"
	"8" "Gain 4 for Neutral current"	
	} {
		radiobutton $gain_cal.f2.rb$p -text $q -variable gain_number -value $p
		pack $gain_cal.f2.rb$p -side top -anchor w -padx 20
	}	
	$gain_cal.f2.rb1 select
	

 
# Phase angle Calibration Frame
#########################################################	 	
	frame 	$phase_angle_cal.f1
	frame 	$phase_angle_cal.f2
	pack 	$phase_angle_cal.f1 -side left -anchor nw
	pack 	$phase_angle_cal.f2 -side left -anchor nw
	
	#label   $phase_angle_cal.f1.l1	-text "Phase Current: " -font {times 10 {bold italic underline}}
	
	label   $phase_angle_cal.f1.l4	-text "Calculated Gain coefficient: "
	entry 	$phase_angle_cal.f2.l4 -textvariable angle_coefficient -width 6
				

	button 	$phase_angle_cal.f1.l5	-text "Calibrate" -command { \
					#puts $gain_number
					
					exit_all_modes
					reinitialize_con
					switch_calibration_mode
					
					
					if { $angle_coefficient > [lindex $coes [expr $phase_gain_number+7]] } {		
					set coeff2 [expr $angle_coefficient-[lindex $coes [expr $phase_gain_number+7]]]
					calibrate_phase_coefficient A$phase_gain_number $coeff2	
					} else {	
					set coeff3 [expr (256-[lindex $coes [expr $phase_gain_number+7]]+$angle_coefficient)]
						calibrate_phase_coefficient A$phase_gain_number $coeff3
					}							
			}	
			
	
			
	for {set i 4} {$i<6} {incr i} {pack $phase_angle_cal.f1.l$i -side top -anchor nw}	
	#label   $phase_angle_cal.f2.l1		
	
	
	
	for {set i 4} {$i<5} {incr i} {pack $phase_angle_cal.f2.l$i -side top -anchor nw}			
	
	foreach {p q } {
	"1" "Gain 1 for Phase current"
	"2" "Gain 2 for Phase current"
	"3" "Gain 3 for Phase current"
	"4" "Gain 4 for Phase current"
	"5" "Gain 1 for Neutral current"
	"6" "Gain 2 for Neutral current"
	"7" "Gain 3 for Neutral current"
	"8" "Gain 4 for Neutral current"	
	} {
		radiobutton $phase_angle_cal.f2.rb$p -text $q -variable phase_gain_number -value $p
		pack $phase_angle_cal.f2.rb$p -side top -anchor w -padx 20
	}	
	$phase_angle_cal.f2.rb1 select


# All-coefficients frame
#########################################################	 	

	frame 	$all_coefficients.f1
	frame 	$all_coefficients.f2
	frame 	$all_coefficients.f3
	frame 	$all_coefficients.f4
	
	pack 	$all_coefficients.f1 -side left -anchor nw	
	pack 	$all_coefficients.f2 -side left -anchor nw	
	pack 	$all_coefficients.f3 -side left -anchor nw	
	pack 	$all_coefficients.f4 -side left -anchor nw	

	
	label   $all_coefficients.f1.l1 	-text "Voltage:" 
	label   $all_coefficients.f1.l2 	-text "Gain, Gain 1, Phase current:" 	 
	label   $all_coefficients.f1.l3 	-text "Gain, Gain 2, Phase current:" 	 
	label   $all_coefficients.f1.l4 	-text "Gain, Gain 3, Phase current:" 	 
	label   $all_coefficients.f1.l5 	-text "Gain, Gain 4, Phase current:" 	 
	label   $all_coefficients.f1.l6 	-text "Gain, Gain 1, Neutral current:" 	 
	label   $all_coefficients.f1.l7 	-text "Gain, Gain 2, Neutral current:" 	 
	label   $all_coefficients.f1.l8 	-text "Gain, Gain 3, Neutral current:" 	 
	label   $all_coefficients.f1.l9 	-text "Gain, Gain 4, Neutral current:" 	 	

	label   $all_coefficients.f3.l0		
	label   $all_coefficients.f3.l1		-text "Phase angle, Gain 1, Phase current:" 	 
	label   $all_coefficients.f3.l2		-text "Phase angle, Gain 2, Phase current:" 	 
	label   $all_coefficients.f3.l3		-text "Phase angle, Gain 3, Phase current:" 	 
	label   $all_coefficients.f3.l4		-text "Phase angle, Gain 4, Phase current:" 	 
	label   $all_coefficients.f3.l5		-text "Phase angle, Gain 1, Neutral current:" 	 
	label   $all_coefficients.f3.l6		-text "Phase angle, Gain 2, Neutral current:" 	 
	label   $all_coefficients.f3.l7		-text "Phase angle, Gain 3, Neutral current:" 	 
	label   $all_coefficients.f3.l8		-text "Phase angle, Gain 4, Neutral current:" 	 
	
	button 	$all_coefficients.f3.l9		-text "Retrieve" -command { \
							# read all the coefficients
							exit_all_modes
							reinitialize_con
							switch_calibration_mode
							
							
							set vol_coe [lindex [read_voltage_coefficient1] 3]
							set coes [read_coefficients]		
							
							
							$all_coefficients.f2.l1 configure -text $vol_coe 														
							
							$all_coefficients.f2.l2 configure -text [lindex $coes 0]
							$all_coefficients.f2.l3 configure -text [lindex $coes 1]
							$all_coefficients.f2.l4 configure -text [lindex $coes 2]
							$all_coefficients.f2.l5 configure -text [lindex $coes 3]
							$all_coefficients.f2.l6 configure -text [lindex $coes 4]
							$all_coefficients.f2.l7 configure -text [lindex $coes 5]														
							$all_coefficients.f2.l8 configure -text [lindex $coes 6]
							$all_coefficients.f2.l9 configure -text [lindex $coes 7]

							$all_coefficients.f4.l1 configure -text [lindex $coes 8 ]
							$all_coefficients.f4.l2 configure -text [lindex $coes 9 ]
							$all_coefficients.f4.l3 configure -text [lindex $coes 10]
							$all_coefficients.f4.l4 configure -text [lindex $coes 11]
							$all_coefficients.f4.l5 configure -text [lindex $coes 12]
							$all_coefficients.f4.l6 configure -text [lindex $coes 13]
							$all_coefficients.f4.l7 configure -text [lindex $coes 14]
							$all_coefficients.f4.l8 configure -text [lindex $coes 15]




							
						}	

	
	for {set i 1} {$i<10} {incr i} {pack $all_coefficients.f1.l$i -side top -anchor nw}	
	for {set i 0} {$i<10} {incr i} {pack $all_coefficients.f3.l$i -side top -anchor nw}	
	                                                                                        
	label   $all_coefficients.f2.l1 	-width 15                              
	label   $all_coefficients.f2.l2 	-width 15
	label   $all_coefficients.f2.l3 	-width 15
	label   $all_coefficients.f2.l4 	-width 15
	label   $all_coefficients.f2.l5 	-width 15
	label   $all_coefficients.f2.l6 	-width 15
	label   $all_coefficients.f2.l7 	-width 15
	label   $all_coefficients.f2.l8 	-width 15
	label   $all_coefficients.f2.l9 	-width 15

	
	                                           
	label   $all_coefficients.f4.l0	
	label   $all_coefficients.f4.l1	 	-width 15
	label   $all_coefficients.f4.l2	 	-width 15
	label   $all_coefficients.f4.l3		-width 15
	label   $all_coefficients.f4.l4		-width 15
	label   $all_coefficients.f4.l5		-width 15
	label   $all_coefficients.f4.l6		-width 15
	label   $all_coefficients.f4.l7		-width 15
	label   $all_coefficients.f4.l8		-width 15
	
	for {set i 1} {$i<10} {incr i} {pack $all_coefficients.f2.l$i -side top -anchor nw}			
	for {set i 0} {$i<9} {incr i} {pack $all_coefficients.f4.l$i -side top -anchor nw}			

#Present Reading frame	
#########################################################
	frame 	$Present_Reading.f1
	frame 	$Present_Reading.f2
	frame 	$Present_Reading.f3
	frame 	$Present_Reading.f4

	pack 	$Present_Reading.f1 -side left -anchor nw
	pack 	$Present_Reading.f2 -side left -anchor nw
	pack 	$Present_Reading.f3 -side left -anchor nw
	pack 	$Present_Reading.f4 -side left -anchor nw
	
	label   $Present_Reading.f1.l1 -text "Voltage:"         
	label   $Present_Reading.f1.l2 -text "Current:" 
 	label   $Present_Reading.f2.l1        
    	label   $Present_Reading.f2.l2 -text "Phase:"         
	label   $Present_Reading.f2.l3 -text "Neutral:"        
	label   $Present_Reading.f2.l4       
	label   $Present_Reading.f3.l1 -text "V" 	
	label   $Present_Reading.f4.l1        
   	label   $Present_Reading.f4.l2 -text "A"       
	label   $Present_Reading.f4.l3 -text "A"        
	label   $Present_Reading.f4.l4        

       	
	label   $Present_Reading.f3.l2 
	label   $Present_Reading.f3.l3    
	 
    	for {set i 1} {$i<2} {incr i} {pack $Present_Reading.f2.l$i -side top -anchor nw}			
	for {set i 1} {$i<4} {incr i} {pack $Present_Reading.f3.l$i -side top -anchor nw}			
                    
        button 	$Present_Reading.f4.l5		-text "Retrieve" -command { \
        #Read all the voltage and current(phase and neutral) values
	exit_all_modes
	reinitialize_con
	switch_calibration_mode
					
	set pres [read_voltage_coefficient1]

 																																																																																																																																																					                                                        							
	$Present_Reading.f2.l1 configure -text [lindex $pres 0]  
	$Present_Reading.f3.l2 configure -text [lindex $pres 1]  
	$Present_Reading.f3.l3 configure -text [lindex $pres 2]  
	                                                       
                  
	      }
	      
	for {set i 1} {$i<3} {incr i} {pack $Present_Reading.f1.l$i -side top -anchor nw}	
	for {set i 1} {$i<5} {incr i} {pack $Present_Reading.f2.l$i -side top -anchor nw}	
	for {set i 1} {$i<4} {incr i} {pack $Present_Reading.f3.l$i -side top -anchor nw}	
        for {set i 1} {$i<6} {incr i} {pack $Present_Reading.f4.l$i -side top -anchor nw}	
	  
	                               
                                                        
                                                        
#########################################################
#
# setting main window title, size, and location
#
wm title . "RSS-SIN Single Phase Energy Meter User Interface"
wm geometry . 600x480+10+10
wm resizable . 0 0

       
     	initialize_con
	switch_program_mode

set 	present_scroll_period [read_scroll_period]
set 	present_unit_code [read_unit_code]
set 	present_int_period [read_md_period]

$scroll_period2.l1 configure -text $present_scroll_period 			
$unit_code2.l1 configure -text $present_unit_code 
$int_period2.l1 configure -text $present_int_period 

set t 		[read_tariffs]
	puts $t 
	set t_slot1 	[lindex $t 0]
	set t_slot2 	[lindex $t 1]
	set t_slot3 	[lindex $t 2]
	set t_slot4 	[lindex $t 3]
	set t_slot5 	[lindex $t 4]
	set t_slot6 	[lindex $t 5]

	set st_slot1 	[lindex $t 7]:[lindex $t 6 ]  
	set st_slot2 	[lindex $t 11]:[lindex $t 10]
	set st_slot3 	[lindex $t 15]:[lindex $t 14]
	set st_slot4 	[lindex $t 19]:[lindex $t 18]
	set st_slot5 	[lindex $t 23]:[lindex $t 22]
	set st_slot6 	[lindex $t 27]:[lindex $t 26]	

	set et_slot1 	[lindex $t 9 ]:[lindex $t 8]
	set et_slot2 	[lindex $t 13]:[lindex $t 12]
	set et_slot3 	[lindex $t 17]:[lindex $t 16]
	set et_slot4 	[lindex $t 21]:[lindex $t 20]
	set et_slot5 	[lindex $t 25]:[lindex $t 24]
	set et_slot6 	[lindex $t 29]:[lindex $t 28]	

set rtc_dat 	  [read_rtc]                                  
set rtc_hh 	  [lindex $rtc_dat 2]                         
set rtc_mm 	  [lindex $rtc_dat 1]                         
set rtc_ss 	  [lindex $rtc_dat 0]                         
set rtc_dd 	  [lindex $rtc_dat 3]                         
set rtc_mon 	  [lindex $rtc_dat 5]                         
set rtc_yy 	20[lindex $rtc_dat 6]                         
set rtc_day_prog  [lindex $DAY_OF_WEEK [lindex $rtc_dat 4]]   
                                                                  
                                                                          
                                                                       



































